package com.bcxin.models.agreement.controller;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.common.constants.GlobalConstants;
import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.core.entity.SysUser;
import com.bcxin.core.service.impl.DownloadService;
import com.bcxin.core.util.SysUserUtils;
import com.bcxin.models.agreement.controller.valid.AgreementRenewGroup;
import com.bcxin.models.agreement.controller.valid.AgreementTerminateGroup;
import com.bcxin.models.agreement.dao.AgreementFileDao;
import com.bcxin.models.agreement.dto.AgreementDto;
import com.bcxin.models.agreement.dto.AgreementFeeDto;
import com.bcxin.models.agreement.entity.Agreement;
import com.bcxin.models.agreement.entity.AgreementFile;
import com.bcxin.models.agreement.service.AgreementFeeService;
import com.bcxin.models.agreement.service.impl.AgreementServiceImpl;
import com.bcxin.models.platform.ConstantURL;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.mybatisplus.toolkit.IdWorker;
import com.bcxin.spring.util.UploadFileUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 订单中心
 * Created by playboy on 2017/6/7.
 */
@Controller
@RequestMapping("${systemPath}/agreementForm")
public class AgreementController extends BaseController {

    /**
     * 订单
     */
    @Autowired
    private AgreementServiceImpl agreementServiceImpl;

    @Resource
    private AgreementFeeService agreementFeeService;

    @Autowired
    private AgreementFileDao agreementFileDao;
    @Autowired
    private DownloadService downloadService;

    /**
     * 交易记录
     */
//    @Autowired
//    private OrderPaymentService orderPaymentService;

    //编辑页
    private static final String ADD = getViewPath("admin/agreement/agreement_form_add");

    //新增
    private static final String APPEND = getViewPath("admin/agreement/agreement_form_append");

    //查看
    private static final String VIEW = getViewPath("admin/agreement/agreement_form_view");

    //终止
    private static final String TERMINATE = getViewPath("admin/agreement/agreement_form_terminate");

    //续签
    private static final String RENEW = getViewPath("admin/agreement/agreement_form_renew");


    //list
    private static final String LIST = getViewPath("admin/agreement/agreement_form_list");

    private static final String INSLIST = getViewPath(ConstantURL.ORDERFORMLISTINS);


    /**
     * 协议列表--页面
     *
     * @param model
     * @return
     */
    @RequiresPermissions("agreement:form:list")
    @RequestMapping("")
    public String listView(Model model) {
        return LIST;
    }

    /**
     * 保险供应商订单列表-- 这个好像没用？
     *
     * @return
     */
    @RequiresPermissions("order:form:insList")
    @RequestMapping("insList")
    public String insList(Model model) {
        SysUser user = SysUserUtils.getUser();
        if (user.getSysCompany() != null && user.getSysCompany().getCompanyType() != null) {
            String url = INSLIST + SysUserUtils.getUser().getSysCompany().getId();
            return redirectTo(url);
        } else {
            return redirectTo(INSLIST + "sup_admin");
        }
    }

    /**
     * 协议列表--数据
     *
     * @param model
     * @param agreement
     * @return
     */
    @RequiresPermissions("agreement:form:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(Model model, AgreementDto agreement) {
        Page page = this.getPage();
        agreementServiceImpl.findPage(page, agreement);
        return jsonPage(page);
    }


    /**
     * 协议列表--查看--修改--删除
     *
     * @param model
     * @param id
     * @return
     * @throws Exception
     */
    //    @RequiresPermissions("agreement:form:form")
    @RequestMapping("/form")
    public String form(Model model, @RequestParam(required = false) Long id) throws Exception {
        String flag = this.getParameter("modify");
        if (id != null) {
//            Map<String,Object> Agreement = agreementServiceImpl.selectById(id);
            AgreementFeeDto agreementFeeDto = new AgreementFeeDto();
            agreementFeeDto.setAgreementId(id);
            if (id == 0) {
                AgreementDto agreementDto = null;
                model.addAttribute("agreementDto", null);
                model.addAttribute("agreementFeeDto", null);
                model.addAttribute("add", flag);
                return APPEND;
            }
            AgreementDto agreementDto = agreementFeeService.agreeDetail(agreementFeeDto);
            model.addAttribute("agreementDto", agreementDto);
            model.addAttribute("agreementFeeDto", JSON.parseArray(JSONObject.toJSONString(agreementDto.getAgreementFeeDto())));
            model.addAttribute("modify", flag);
            if ("view".equalsIgnoreCase(flag)) {
                return VIEW;
            }
        }
        return ADD;
    }

    /**
     * 终止协议页面
     *
     * @param model
     * @param id
     * @return
     */
    @RequestMapping("/form/terminate")
    public String terminateView(Model model, @RequestParam(required = false) Long id) {
        Agreement agreementDto = agreementServiceImpl.selectById(id);
        model.addAttribute("agreementDto", agreementDto);
        return TERMINATE;
    }

    /**
     * 续签协议页面
     *
     * @param model
     * @param id
     * @return
     */
    @RequestMapping("/form/renew")
    public String renewView(Model model, @RequestParam(required = false) Long id) {
        Agreement agreementDto = agreementServiceImpl.selectById(id);
        model.addAttribute("agreementDto", agreementDto);
        return RENEW;
    }

    /**
     * 终止协议
     */
    @RequestMapping("/terminate")
    @ResponseBody
    public AjaxResult terminate(@RequestBody @Validated(value = {AgreementTerminateGroup.class}) AgreementDto dto, BindingResult bindingResult) {
        try {
            if (bindingResult.hasErrors()) {
                String errorInfo = "";
                List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
                for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                    errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";
                }
                throw new RuntimeException(errorInfo);//返回校验错误
            }
            agreementServiceImpl.terminate(dto);
        } catch (Exception e) {
            return fail(false, "续签失败:" + e.getMessage());
        }
        return success(true, "续签成功");
    }

    /**
     * 续签
     */
    @RequestMapping("/renew")
    @ResponseBody
    public AjaxResult renew(@RequestBody @Validated(value = {AgreementRenewGroup.class}) AgreementDto dto, BindingResult bindingResult) {
        try {
            if (bindingResult.hasErrors()) {
                String errorInfo = "";
                List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
                for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                    errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";

                }
                throw new RuntimeException(errorInfo);//返回校验错误
            }
            // 加几个校验，endDate协议时间必须大于今天，且必须大于旧有时间
//            Date date = DateUtil.parse(dto.getEndDate());
            Date today = DateUtil.parse(DateUtil.today());
            if (dto.getEndDate().getTime() < today.getTime()) {
                throw new Exception("续签时间必须大于当前时间");
            }
            agreementServiceImpl.renew(dto);
        } catch (Exception e) {
            return fail(false, "续签失败:" + e.getMessage());
        }
        return success(true, "续签成功");

    }

    /**
     * 图片上传
     *
     * @param files
     * @return
     */
    @RequestMapping(value = "/uploadAll")
    @ResponseBody
    public AjaxResult uploadHeadPortraitAllFormat(@RequestParam("files") MultipartFile[] files) {
        List<String> resultData = new ArrayList<>();
        for (MultipartFile file : files) {
            if (null == file) {
                return success(false);
            }
            SysUser user = SysUserUtils.getUser();
            String fileName = UploadFileUtils.rename(file);
            String path = GlobalConstants.getUserFlie() + File.separator + user.getId().toString() + File.separator + "file";
            File targetFile = new File(path, fileName);
            if (!targetFile.exists()) {
                targetFile.mkdirs();
            }
//            CommonsMultipartFile cf= (CommonsMultipartFile)file;
//            DiskFileItem fi = (DiskFileItem)cf.getFileItem();
            //保存
            try {
                file.transferTo(targetFile);
            } catch (Exception e) {
                e.printStackTrace();
            }
            //存agreementFile表
            Long id = IdWorker.getId();
            AgreementFile agreementFile = new AgreementFile();
//            agreementFile.setId(id);
            agreementFile.setFileId(id.toString());
            agreementFile.setFileName(file.getOriginalFilename());
            agreementFile.setFileUrl(user.getId().toString() + "/file/" + fileName);
            agreementFileDao.insertSelective(agreementFile);
            resultData.add(agreementFile.getId().toString());
        }
        return json(true, String.join(",", resultData));
//        List<String> result = Arrays.asList(str.split(",")); todo

    }

    /**
     * 文件下载控制器
     *
     * @param key  文件名
     * @param req
     * @param resp
     * @throws IOException
     * @throws IOException
     */
    @RequestMapping(value = "/download/{key}", method = RequestMethod.GET)
    public void download(
            @PathVariable(value = "key") String key,
//            @RequestParam(value = "name", required = false) String name,
            HttpServletRequest req, HttpServletResponse resp) throws IOException {
        String fileId = key;
        if (fileId == null || fileId.trim().length() == 0) {
            resp.reset();
            resp.setContentType("text/plain;charset=utf-8");
            PrintWriter writer = resp.getWriter();
            writer.write("error:can't get the file name! 不能获取文件名");
            writer.flush();
            return;
        }
        AgreementFile agreementFile = agreementFileDao.selectByPrimaryKey(Long.valueOf(fileId));
        downloadService.downloadService(agreementFile.getFileUrl(), req, resp);

    }

}
