package com.bcxin.models.agreement.controller;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.models.agreement.controller.valid.AgreementFeeSaveGroup;
import com.bcxin.models.agreement.dto.AgreementDto;
import com.bcxin.models.agreement.dto.AgreementFeeDto;
import com.bcxin.models.agreement.entity.AgreementFee;
import com.bcxin.models.agreement.service.AgreementFeeService;
import com.bcxin.models.product.service.ProductService;
import com.bcxin.mybatisplus.plugins.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static java.util.stream.Collectors.groupingBy;

/**
 * 协议和产品和费用比例(AgreementFee)表控制层
 *
 * @author makejava
 * @since 2020-02-11 13:51:24
 */
@RestController
@RequestMapping("${systemPath}/agreementFee")
public class AgreementFeeController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private AgreementFeeService agreementFeeService;

    @Autowired
    ProductService productService;

    /**
     * 通过主键查询单条数据-- 这条没用
     *
     * @return 单条数据
     */
//    @GetMapping("selectOne")
    @Deprecated
    @RequestMapping(value = "/selectOne", method = RequestMethod.GET)
    @ResponseBody
    public AgreementDto selectOne(@RequestBody AgreementFeeDto dto) {
        try {
            return this.agreementFeeService.agreeDetail(dto);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 新增协议
     *
     * @author wangjianjun
     * @date 2020/3/9
     */
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult saveNew(@RequestBody @Validated(value = {AgreementFeeSaveGroup.class}) AgreementDto dto, BindingResult bindingResult) {
        try {
            if (bindingResult.hasErrors()) {
                String errorInfo = "";
                List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
                for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                    errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";

                }
                throw new RuntimeException(errorInfo);//返回校验错误
            }
            //将所有idAndType覆盖分解为productId和productType
            dto.setAgreementFeeDto(dto.getAgreementFeeDto().stream().filter(s -> {
                String productId = s.getIdAndType().split("_")[1];
                String productType = s.getIdAndType().split("_")[0];
                s.setProductId(productId);
                s.setProductType(productType);
                s.setAgreementId(dto.getId());
                //1，保存协议时，如果计费方式为固定金额，将固定金额赋值给费用比例。
                if (AgreementFee.CALC_METHOD_FIXED.equalsIgnoreCase(s.getCalcMethod())) {
                    s.setFeeRate(s.getFixedAmount());
                }
                return true;
            }).collect(Collectors.toList()));
            List<AgreementFeeDto> rangeList = dto.getAgreementFeeDto().stream().filter(r -> {
                if((r.getTaxScopeStart()==null)||(r.getTaxScopeEnd()==null)){
                    throw new RuntimeException("价税范围不能为空");
                }
                if(r.getFixedCost()==null){
                    throw new RuntimeException("固定成本不能为空");
                }
                if(r.getFixedAmount()==null){
                    throw new RuntimeException("固定金额不能为空");
                }
                if(r.getFeeRate()==null){
                    throw new RuntimeException("费用比例不能为空");
                }
                if (r.getTaxScopeEnd().compareTo(new BigDecimal("0")) == 0) {
                    r.setTaxScopeEnd(new BigDecimal("999999999"));
                }
                return true;
            }).collect(Collectors.toList());
            /**范围校验开始**/
            //统计每个productId的数量
            Map<String, Long> result =
                    rangeList.stream().map(AgreementFee::getProductId).collect(
                            groupingBy(
                                    Function.identity(), Collectors.counting()
                            )
                    );
            Map<String, List<AgreementFeeDto>> groupByList = rangeList.stream().collect(groupingBy(AgreementFee::getProductId));
            rangeList.stream().filter(s -> {
                if (result.containsKey(s.getProductId())) {
                    if (result.get(s.getProductId()) > 1) {
                        List<AgreementFeeDto> list = groupByList.get(s.getProductId());
                        int n = list.size();
                        BigDecimal[][] board = new BigDecimal[n][2];
                        IntStream.range(0, n).forEach(j -> {
                            board[j][0] = list.get(j).getTaxScopeStart();
                            board[j][1] = list.get(j).getTaxScopeEnd();
                        });
                        if (validInterval(board)) {
                            throw new RuntimeException("范围重叠");
                        }
                    }
                }
                return true;
            }).collect(Collectors.toList());
            /**范围校验结束**/
            this.agreementFeeService.agreeDetailAdded(dto);
        } catch (Exception e) {
            e.printStackTrace();
            return success(false, e.getMessage());
        }
        return success(true);
    }

    /**
     * 编辑协议
     *
     * @author wangjianjun
     * @date 2020/3/9
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult save(@RequestBody @Validated(value = {AgreementFeeSaveGroup.class}) AgreementDto dto, BindingResult bindingResult) {
        try {
            if (bindingResult.hasErrors()) {
                String errorInfo = "";
                List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
                for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                    errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";

                }
                throw new RuntimeException(errorInfo);//返回校验错误
            }
            //将所有idAndType覆盖分解为productId和productType
            dto.getAgreementFeeDto().stream().filter(s -> {
                String productId = s.getIdAndType().split("_")[1];
                String productType = s.getIdAndType().split("_")[0];
                s.setProductId(productId);
                s.setProductType(productType);
                s.setAgreementId(dto.getId());
                //1，保存协议时，如果计费方式为固定金额，将固定金额赋值给费用比例。
                if (AgreementFee.CALC_METHOD_FIXED.equalsIgnoreCase(s.getCalcMethod())) {
                    s.setFeeRate(s.getFixedAmount());
                }
                return true;
            }).collect(Collectors.toList());
            List<AgreementFeeDto> rangeList = dto.getAgreementFeeDto();
            /**范围校验开始**/
            rangeList.stream().filter(r -> {
                if (r.getTaxScopeEnd().compareTo(new BigDecimal("0")) == 0) {
                    r.setTaxScopeEnd(new BigDecimal("999999999"));
                }
                return false;
            }).collect(Collectors.toList());
            //统计每个productId的数量
            Map<String, Long> result =
                    rangeList.stream().map(AgreementFee::getProductId).collect(
                            groupingBy(
                                    Function.identity(), Collectors.counting()
                            )
                    );
            Map<String, List<AgreementFeeDto>> groupByList = rangeList.stream().collect(groupingBy(AgreementFee::getProductId));
            rangeList.stream().filter(s -> {
                if (result.containsKey(s.getProductId())) {
                    if (result.get(s.getProductId()) > 1) {
                        List<AgreementFeeDto> list = groupByList.get(s.getProductId());
                        int n = list.size();
                        BigDecimal[][] board = new BigDecimal[n][2];
                        IntStream.range(0, n).forEach(j -> {
                            board[j][0] = list.get(j).getTaxScopeStart();
                            board[j][1] = list.get(j).getTaxScopeEnd();
                        });
                        if (validInterval(board)) {
                            throw new RuntimeException("范围重叠");
                        }
                    }
                }
                return true;
            }).collect(Collectors.toList());
            /**范围校验结束**/
            this.agreementFeeService.agreeDetailSave(dto);
        } catch (Exception e) {
            e.printStackTrace();
            return success(false, e.getMessage());
        }
        return success(true);
    }

    @RequestMapping(value = "/listData", method = RequestMethod.GET)
//    @ResponseBody
    public String listData(Model model, Long agreementId) throws Exception {
        Page<AgreementFeeDto> page = this.getPage();
//        page.setSize(50);
        AgreementFeeDto agreementFee = new AgreementFeeDto();
        agreementFee.setAgreementId(agreementId);
        agreementFeeService.findList(page, agreementFee);

        //  展示数据的时候。如果计费方式为固定金额，将费用比例设置为0。若为比例，将费用比例设置为0。
        page.getRecords().stream().filter(s -> {
            if (AgreementFee.CALC_METHOD_RATIO.equalsIgnoreCase(s.getCalcMethod())) {
                s.setFixedAmount(new BigDecimal(0));
            } else {
                s.setFeeRate(new BigDecimal(0));
            }
            return true;
        }).collect(Collectors.toList());
        String result = jsonPage(page);
        return result;
    }

    @RequestMapping(value = "/delete", method = RequestMethod.DELETE)
    @ResponseBody
    public AjaxResult save(String ids) {
        List a = Arrays.asList(ids.split(","));
        agreementFeeService.deleteBatchIds(Arrays.asList(ids.split(",")));
        return success(true);
    }

    /**
     * 判断一个协议里多个同种商品的价税范围是否重叠
     * 1,提取出数组
     * 2，按最左边的数字排序
     * 3，用每个右边的数字和上一个左边的数字进行比较
     *
     * @author wangjianjun
     * @date 2020/3/10
     */
    private boolean validInterval(BigDecimal[][] bigs) {
        /**
         * 左排序
         */
        Arrays.sort(bigs, new Comparator<BigDecimal[]>() {
            @Override
            public int compare(BigDecimal[] o1, BigDecimal[] o2) {
                if (o1[0].compareTo(o2[0]) == 1) {
                    return 1;
                } else if (o1[0].compareTo(o2[0]) == -1) {
                    return -1;
                } else {
                    return 0;
                }
            }
        });
        /**
         *比较
         */
        int i = 0;
        while (bigs.length > i + 1) {
            if (bigs[i + 1][0].compareTo(bigs[i][1]) < 1) {//compareTo大于时，返回 1
                return true;
//                throw new Exception("范围重叠");
            }
            i++;
        }
        return false;
    }

}