package com.bcxin.models.agreement.entity;

import com.bcxin.models.agreement.SimpleEntity;
import com.bcxin.models.agreement.controller.valid.AgreementFeeSaveGroup;
import com.bcxin.models.agreement.controller.valid.AgreementRenewGroup;
import com.bcxin.models.agreement.controller.valid.AgreementTerminateGroup;
import com.bcxin.mybatisplus.annotations.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.NotBlank;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * 协议Entity
 * @author wjj
 * @version 2020-02-05
 */
@Data
//@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
@TableName("agreement")
public class Agreement extends SimpleEntity<Agreement> {

	/**
	 * 协议代码
	 */
	private String code;
	/**
	 * 公司id,合作机构id
	 */
	@NotNull(message = "合作机构不能为空",groups = {AgreementFeeSaveGroup.class})
	private Long companyId;
	/**
	 * 合作伙伴类型 1:供应商 0:其它(资源方,或者分销商)
	 */
	private String companyType;
	/**
	 * 协议类型，agree_type，4资源方协议，2分销商协议，1供应商协议
	 */
	@NotBlank(message = "协议类型不能为空",groups = {AgreementFeeSaveGroup.class})
	private String type;
	/**
	 * 协议名称
	 */
	@NotBlank(message = "协议名称不能为空",groups = {AgreementFeeSaveGroup.class})
	private String name;
	/**
	 * 协议签订日期
	 */
	@JsonFormat(pattern="yyyy-MM-dd",timezone = "GMT+8")
	@DateTimeFormat(pattern="yyyy-MM-dd")
	@NotNull(message = "签订日期不能为空",groups = {AgreementFeeSaveGroup.class})
	private Date odate;
	/**
	 * 协议有效开始日期
	 */
	@NotNull(message = "协议开始日期不能为空",groups = {AgreementFeeSaveGroup.class})
	@JsonFormat(pattern="yyyy-MM-dd",timezone = "GMT+8")
	@DateTimeFormat(pattern="yyyy-MM-dd")
	private Date startDate;
	/**
	 * 协议有效结束日期
	 */
	@JsonFormat(pattern="yyyy-MM-dd",timezone = "GMT+8")
	@DateTimeFormat(pattern="yyyy-MM-dd")
	@NotNull(message = "协议结束日期不能为空",groups = {AgreementFeeSaveGroup.class, AgreementRenewGroup.class})
	private Date endDate;
	/**
	 * 协议状态,未生效0，执行中1,已终止2,已完结3
	 */
//	private String status;
	/**
	 * 暂时废弃，结算周期，1每月结算，2每季结算
	 */
//	@NotBlank(message = "结算周期不能为空",groups = {AgreementFeeSaveGroup.class})
	private String feeCycle;
	/**
	 * 是否终止协议,todo 这个字段似乎没啥用，应该可以去掉，暂时保留
	 */
	private String stopFlag;
	/**
	 * 终止原因
	 */
	@NotBlank(message = "终止原因不能为空",groups = {AgreementTerminateGroup.class})
	private String stopAgreeReason;
	/**
	 * 协议文件
	 */
	@NotBlank(message = "协议文件不能为空",groups = {AgreementRenewGroup.class})
	private String picture;

}