package com.bcxin.models.agreement.service.impl;

import com.bcxin.core.base.service.impl.BaseServiceImpl;
import com.bcxin.models.agreement.dao.AgreementDao;
import com.bcxin.models.agreement.dao.AgreementFeeDao;
import com.bcxin.models.agreement.dao.AgreementFileDao;
import com.bcxin.models.agreement.dto.AgreementDto;
import com.bcxin.models.agreement.dto.AgreementFeeDto;
import com.bcxin.models.agreement.entity.Agreement;
import com.bcxin.models.agreement.entity.AgreementFee;
import com.bcxin.models.agreement.entity.AgreementFile;
import com.bcxin.models.agreement.service.AgreementFeeService;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 协议和产品和费用比例(AgreementFee)表服务实现类
 *
 * @author makejava
 * @since 2020-02-11 13:51:24
 */
@Service
public class AgreementFeeServiceImpl extends BaseServiceImpl<AgreementFeeDao, AgreementFee> implements AgreementFeeService {
    @Resource
    private AgreementFeeDao agreementFeeDao;
    @Resource
    private AgreementDao agreementDao;
    @Resource
    private AgreementFileDao agreementFileDao;

    @Override
    public AgreementDto agreeDetail(AgreementFeeDto agreementFeeDto) throws Exception {
        Agreement agreement = agreementDao.selectByPrimaryKey(agreementFeeDto.getAgreementId());
        AgreementDto agreementDto = new AgreementDto();
        String fileName = "";
        if (StringUtils.isNotEmpty(agreement.getPicture())) {
            AgreementFile af = agreementFileDao.selectByPrimaryKey(Long.valueOf(agreement.getPicture()));
            if (af != null)
                fileName = af.getFileName();
        }
        BeanUtils.copyProperties(agreement, agreementDto);
        agreementDto.setFileName(fileName);
//        List<AgreementFeeDto> result = agreementFeeDao.agreeDetail(agreementFeeDto);
//        agreementDto.setAgreementFeeDto(result);
//        return new ObjectMapper().writeValueAsString(agreementDto);
        return agreementDto;
    }

    @Override
    public List<AgreementFeeDto> agreeFeeDetail(AgreementFeeDto agreementFeeDto) throws Exception {
        List<AgreementFeeDto> list = agreementFeeDao.agreeDetail(agreementFeeDto);
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String agreeDetailSave(AgreementDto agreementDto) {
        //编辑模式
        agreementDao.updateByPrimaryKeySelective(agreementDto);
        //先清掉agreementFee，再重新添加，原ID要插入。
        //这里增加一个saveOrUpdate方法
        agreementFeeDao.delete(new EntityWrapper<AgreementFee>().eq("agreement_id", agreementDto.getId()));
        agreementFeeDao.insertBatch(agreementDto.getAgreementFeeDto());

        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String agreeDetailAdded(AgreementDto agreementDto) throws Exception {
        //将所有idAndType覆盖分解为productId和productType
        agreementDto.setAgreementFeeDto(agreementDto.getAgreementFeeDto().stream().filter(s -> {
            String productId = s.getIdAndType().split("_")[1];
            String productType = s.getIdAndType().split("_")[0];
            s.setProductId(productId);
            s.setProductType(productType);
            s.setAgreementId(agreementDto.getId());
            //1，保存协议时，如果计费方式为固定金额，将固定金额赋值给费用比例。（0.5小时）
            if (AgreementFee.CALC_METHOD_FIXED.equalsIgnoreCase(s.getCalcMethod())) {
                s.setFeeRate(s.getFixedAmount());
            } else {
                s.setFixedAmount(new BigDecimal(0));//比例时，固定金额置为0.
            }

            return true;
        }).collect(Collectors.toList()));

        if (!agreementDao.selectCount(new EntityWrapper<Agreement>().eq("company_id", agreementDto.getCompanyId()).eq("status", "1")).equals(0)) {
            throw new Exception("协议已存在，并且未过期，不能重复添加");
        }
//        Long id = IdWorker.getId();
//        agreementDto.setId(id);
        agreementDao.insertSelective(agreementDto);
        agreementFeeDao.insertBatchWithoutId(agreementDto.getAgreementFeeDto().stream().filter(s -> {
            s.setAgreementId(agreementDto.getId());
            return true;
        }).collect(Collectors.toList()));
        return null;
    }

    @Override
    public Page<AgreementFeeDto> findList(Page<AgreementFeeDto> page, AgreementFeeDto agreementFee) {
        page.setRecords(dao.findList(page, agreementFee));
        return page;
    }


//    /**
//     * 通过ID查询单条数据
//     *
//     * @param id 主键
//     * @return 实例对象
//     */
//    @Override
//    public AgreementFee queryById(String id) {
//        return this.agreementFeeDao.queryById(id);
//    }
//
//    /**
//     * 查询多条数据
//     *
//     * @param offset 查询起始位置
//     * @param limit 查询条数
//     * @return 对象列表
//     */
//    @Override
//    public List<AgreementFee> queryAllByLimit(int offset, int limit) {
//        return this.agreementFeeDao.queryAllByLimit(offset, limit);
//    }
//
//    /**
//     * 新增数据
//     *
//     * @param agreementFee 实例对象
//     * @return 实例对象
//     */
//    @Override
//    public AgreementFee insert(AgreementFee agreementFee) {
//        this.agreementFeeDao.insert(agreementFee);
//        return agreementFee;
//    }
//
//    /**
//     * 修改数据
//     *
//     * @param agreementFee 实例对象
//     * @return 实例对象
//     */
//    @Override
//    public AgreementFee update(AgreementFee agreementFee) {
//        this.agreementFeeDao.update(agreementFee);
//        return this.queryById(agreementFee.getId());
//    }
//
//    /**
//     * 通过主键删除数据
//     *
//     * @param id 主键
//     * @return 是否成功
//     */
//    @Override
//    public boolean deleteById(String id) {
//        return this.agreementFeeDao.deleteById(id) > 0;
//    }
}