package com.bcxin.models.bill.controller;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.models.bill.controller.valid.BillSaveGroup;
import com.bcxin.models.bill.dto.BillDto;
import com.bcxin.models.bill.service.BillService;
import com.bcxin.models.platform.ConstantURL;
import com.bcxin.mybatisplus.plugins.Page;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 订单中心
 *
 * @author playboy
 * @date 2017/6/7
 */
@Controller
@RequestMapping("${systemPath}/billForm")
public class NewBillController extends BaseController {

    /**
     * 订单
     */
    @Autowired
    private BillService billService;

    //add大抵是详情页面的意思
    private static final String ADD = getViewPath("admin/bill/bill_form_add");

    //view
    private static final String LIST = getViewPath("admin/bill/bill_form_list");

    private static final String INSLIST = getViewPath(ConstantURL.ORDERFORMLISTINS);


    /**
     * 分销商订单列表
     *
     * @param model
     * @return
     */
    @RequiresPermissions("bill:form:list")
    @RequestMapping("")
    public String listView(Model model) {
        return LIST;
    }


    @RequiresPermissions("bill:form:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(BillDto dto){
//        if(settleId!=null) {
//            dto.setSettleId(settleId);
//        }
//        Page page =this.getPage();
        Page<Map<String,String>> page =this.getPage();
        billService.findPage(page,dto);
        return jsonPage(page);
    }


    @RequestMapping("/form")
    public String form(Model model, @RequestParam(required = false) Long id) throws Exception {
        String flag = this.getParameter("modify");
        if (id != null) {
            BillDto billDto=new BillDto();
            billDto.setId(id);
            BillDto billDtoResult = billService.detail(billDto);
            model.addAttribute("billForm", billDtoResult);
        }
        model.addAttribute("modify", flag);
        return ADD;
    }

    /**
     * 对账功能
     * @param dto
     * @return
     */

    @RequestMapping(value = "/save",method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult save(@RequestBody @Validated(value = {BillSaveGroup.class}) BillDto dto, BindingResult bindingResult ) {
        try {
            if (bindingResult.hasErrors()) {
                String errorInfo = "";
                List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
                for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                    errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";

                }
                throw new RuntimeException(errorInfo);//返回校验错误
            }
            dto.setStatus("1");
            billService.updateById(dto);
        } catch (Exception e) {
            e.printStackTrace();
            return success(false);
        }
        return success(true);
    }

}
