package com.bcxin.models.fee.entity;

import com.bcxin.core.base.entity.DataEntity;
import com.bcxin.mybatisplus.annotations.TableField;
import com.bcxin.mybatisplus.annotations.TableName;
import com.bcxin.util.toolbox.DateUtil;
import org.hibernate.validator.constraints.NotBlank;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 结算单
 * @author 刘金炬
 */
@TableName("fee_settlement")
public class FeeSettlement extends DataEntity<FeeSettlement> {

    private static final long serialVersionUID = 1L;

    /**
     * 结算日期
     */
    @NotNull(message = "结算日期不能为空")
    @TableField("odate")
    private Date odate;

    /**
     * 结算单号
     */
    @TableField("code")
    private String code;

    /**
     * 结算金额
     */
    @NotNull(message = "结算金额不能为空")
    @TableField("fee_fcy")
    private BigDecimal feeFcy;

    /**
     * 订单数量
     */
    @TableField("order_form_con")
    private Integer orderFormCon;

    /**
     * 开始日期
     */
    @NotNull(message = "开始日期不能为空")
    @TableField("start_date")
    private Date startDate;

    /**
     * 结束日期
     */
    @NotNull(message = "结束日期不能为空")
    @TableField("end_date")
    private Date endDate;

    /**
     * 红冲字段
     */
    @TableField("ids")
    private String ids;

    /**
     * 币种
     */
    @NotBlank(message = "币种不能为空")
    @TableField("currency")
    private String currency;

    /**
     * 费用类型
     */
    @NotNull(message = "费用类型不能为空")
    @TableField("type")
    private Integer type;

    /**
     * 订单号
     */
    @NotNull(message = "订单号不能为空")
    @TableField("order_no")
    private String orderNo;

    @NotNull(message = "订单费率")
    @TableField("order_fee")
    private BigDecimal orderFee=BigDecimal.ZERO;

    /**
     * 产品id
     */
    @NotNull(message = "产品id不能为空")
    @TableField("product_id")
    private String productId;

    /**
     * 扣除税金
     */
    @TableField("tax_money")
    private String taxMoney;

    /**
     * 代扣成本
     */
    @TableField("withholding_costs")
    private String withholdingCosts;

    /**
     * 平台管理费
     */
    @TableField("platform_management_fee")
    private String platformManagementFee;

    /**
     * 实得佣金
     */
    @NotNull(message = "实得佣金不能为空")
    @TableField("actual_brokerage")
    private String actualBrokerage;

    /**
     * 已结算佣金
     */
    @TableField("settled_commission")
    private String settledCommission;

    /**
     * 已发放佣金
     */
    @TableField("commission_paid")
    private String commissionPaid;

    /**
     * 类型
     */
    @NotNull(message = "类型：1:供应商，2分销商，4.资源方不能为空")
    @TableField("sys_type")
    private String sysType;

    /**
     * 合作方id
     */
    @NotNull(message = "合作方id 不能为空")
    @TableField("sys_company_id")
    private Long sysCompanyId;
    /**
     * 上级分销商 id
     */
    @TableField("parent_id")
    private Long parentID;

    @NotBlank
    @TableField("hxxt_id")
    private String hxxtId;


    public Long getParentID() {
        return parentID;
    }

    public void setParentID(Long parentID) {
        this.parentID = parentID;
    }

    public String getProductId() {
        return productId;
    }

    public void setProductId(String productId) {
        this.productId = productId;
    }

    public String getTaxMoney() {
        return taxMoney;
    }

    public void setTaxMoney(String taxMoney) {
        this.taxMoney = taxMoney;
    }

    public String getWithholdingCosts() {
        return withholdingCosts;
    }

    public void setWithholdingCosts(String withholdingCosts) {
        this.withholdingCosts = withholdingCosts;
    }

    public String getPlatformManagementFee() {
        return platformManagementFee;
    }

    public void setPlatformManagementFee(String platformManagementFee) {
        this.platformManagementFee = platformManagementFee;
    }

    public String getActualBrokerage() {
        return actualBrokerage;
    }

    public void setActualBrokerage(String actualBrokerage) {
        this.actualBrokerage = actualBrokerage;
    }

    public String getSettledCommission() {
        return settledCommission;
    }

    public void setSettledCommission(String settledCommission) {
        this.settledCommission = settledCommission;
    }

    public String getCommissionPaid() {
        return commissionPaid;
    }

    public void setCommissionPaid(String commissionPaid) {
        this.commissionPaid = commissionPaid;
    }

    public String getSysType() {
        return sysType;
    }

    public void setSysType(String sysType) {
        this.sysType = sysType;
    }

    public Long getSysCompanyId() {
        return sysCompanyId;
    }

    public void setSysCompanyId(Long sysCompanyId) {
        this.sysCompanyId = sysCompanyId;
    }

    public Date getOdate() {
        return odate;
    }

    public void setOdate(Date odate) {
        this.odate = odate;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public BigDecimal getFeeFcy() {
        return feeFcy;
    }

    public void setFeeFcy(BigDecimal feeFcy) {
        this.feeFcy = feeFcy;
    }

    public Integer getOrderFormCon() {
        return orderFormCon;
    }

    public void setOrderFormCon(Integer orderFormCon) {
        this.orderFormCon = orderFormCon;
    }

    public Date getStartDate() {

        if(startDate!=null){
            startDate = DateUtil.parseDate(DateUtil.format(startDate,"yyyy-MM-dd 00:00:00"));
        }
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Date getEndDate() {
        if(endDate!=null){
            endDate = DateUtil.parseDate(DateUtil.format(endDate,"yyyy-MM-dd 23:59:59"));
        }
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getIds() {
        return ids;
    }

    public void setIds(String ids) {
        this.ids = ids;
    }

    public String getCurrency() {
        return currency;
    }

    public void setCurrency(String currency) {
        this.currency = currency;
    }

    public Integer getType() {
        return type;
    }

    public void setType(Integer type) {
        this.type = type;
    }


    public String getOrderNo() {
        return orderNo;
    }

    public void setOrderNo(String orderNo) {
        this.orderNo = orderNo;
    }

    public String getHxxtId() {
        return hxxtId;
    }

    public void setHxxtId(String hxxtId) {
        this.hxxtId = hxxtId;
    }

    public BigDecimal getOrderFee() {
        return orderFee;
    }

    public void setOrderFee(BigDecimal orderFee) {
        this.orderFee = orderFee;
    }
}