package com.bcxin.models.fee.web;

import com.bcxin.core.base.web.BaseController;
import com.bcxin.core.entity.SysCompany;
import com.bcxin.core.entity.SysUser;
import com.bcxin.core.service.SysCompanyService;
import com.bcxin.core.util.SysUserUtils;
import com.bcxin.models.fee.entity.FeeSettlement;
import com.bcxin.models.fee.service.FeeSettlementService;
import com.bcxin.mybatisplus.plugins.Page;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.Map;

/**
 * 账单中心  管理员内部看的
 * Created by liyongqiang on 2017/7/28.
 */
@Controller
@RequestMapping("${systemPath}/fee/company")
public class FeeCompanyController extends BaseController {

    @Autowired
    private FeeSettlementService feeSettlementService;

    @Autowired
    private SysCompanyService sysCompanyService;

    //分销商账单列表
    private static final String DISTRIBUTOR_LIST = getViewPath("admin/fee/company/distributor_list");
    //分销商下级账单列表
    private static final String SUBDISTRIBUTOR_LIST = getViewPath("admin/fee/company/subdistributor_list");

    //分销商详细信息
    private static final String DISTRIBUTOR_DETAIL = getViewPath("admin/fee/company/distributor_detatil");

    //资源方账单列表
    private static final String RESOURCE_SIDE_LIST = getViewPath("admin/fee/company/resource_side_list");

    //供应商账单列表
    private static final String SUPPLY_LIST = getViewPath("admin/fee/company/supplier_list");



    @RequiresPermissions("fee:manager:view")
    @RequestMapping("/initFeeList")
    public String initFeeList(Model model){
        SysUser user = SysUserUtils.getUser();
        SysCompany company =sysCompanyService.selectById(user.getSysCompany().getId());
        model.addAttribute("company",company);
        if(2==company.getCompanyType()){
            //分销商
            return DISTRIBUTOR_LIST;
        }
        if(4==company.getCompanyType()){
            //资源方
            return RESOURCE_SIDE_LIST;
        }
        if(1==company.getCompanyType()){
            //供应商
            return SUPPLY_LIST;
        }
        return  null;
    }
    @RequiresPermissions("fee:submanager:view")
    @RequestMapping("/initDistributorListByID")
    public String InitDistributorList(Model model){
        Map<String,String> map =feeSettlementService.getTotalFee(SysCompany.COMPANY_TYPE_DISTRIBUTOR.toString());
        model.addAttribute("map",map);
        return SUBDISTRIBUTOR_LIST;
    }
    @RequestMapping("/getDistributorListData")
    @ResponseBody
    public String getDistributorListData(Model model, FeeSettlement feeSettlement){
        SysUser user = SysUserUtils.getUser();
        Page<Map<String,String>> page =this.getPage();
        feeSettlement.setSysType(SysCompany.COMPANY_TYPE_DISTRIBUTOR.toString());
        feeSettlement.setSysCompany(user.getSysCompany());
        feeSettlementService.findOnePage(page,feeSettlement,feeSettlement.getSysType());
        return jsonPage(page);
    }
    @RequestMapping("/getDistributorListDataByID")
    @ResponseBody
    public String getDistributorListDataByID(Model model, FeeSettlement feeSettlement){
        SysUser user = SysUserUtils.getUser();
        Page<Map<String,String>> page =this.getPage();
        feeSettlement.setParentID(user.getSysCompany().getId());
        feeSettlement.setSysType(SysCompany.COMPANY_TYPE_DISTRIBUTOR.toString());
        feeSettlement.setSysCompany(user.getSysCompany());
        feeSettlementService.findPage(page,feeSettlement,feeSettlement.getSysType());
        return jsonPage(page);
    }

    @RequestMapping("/getResourceSideListData")
    @ResponseBody
    public String findResourceSideList(Model model, FeeSettlement feeSettlement){
        SysUser user = SysUserUtils.getUser();
        Page<Map<String,String>> page =this.getPage();
        feeSettlement.setSysType(SysCompany.COMPANY_TYPE_CHANNEL.toString());
        feeSettlement.setSysCompany(user.getSysCompany());
        feeSettlementService.findOnePage(page,feeSettlement,feeSettlement.getSysType());
        return jsonPage(page);
    }


    @RequestMapping("/getSupplyListData")
    @ResponseBody
    public String getSupplyListData(Model model, FeeSettlement feeSettlement){
        SysUser user = SysUserUtils.getUser();
        Page<Map<String,String>> page =this.getPage();
        feeSettlement.setSysType(SysCompany.COMPANY_TYPE_SUPPLIER.toString());
        feeSettlement.setSysCompany(user.getSysCompany());
        feeSettlementService.findOnePage(page,feeSettlement,feeSettlement.getSysType());
        return jsonPage(page);
    }


    /**
     * 查询明细
     * @param model
     * @param id
     * @return
     */
    @RequestMapping("/getDetatilList/{id}")
    public String getDetatilList(Model model, @PathVariable Long id){
        FeeSettlement fee= feeSettlementService.selectById(id);
        Map<String,String> feeMap=feeSettlementService.getFeeDetaitil(id);
        if(fee==null){
            return null;
        }
        model.addAttribute("fee",fee);
        model.addAttribute("feeMap",feeMap);
        return   DISTRIBUTOR_DETAIL;

    }

}
