package com.bcxin.models.order.entity;

import com.bcxin.core.base.entity.DataEntity;
import com.bcxin.core.entity.SysCompany;
import com.bcxin.mybatisplus.annotations.TableField;
import com.bcxin.mybatisplus.annotations.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.hibernate.validator.constraints.NotBlank;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 订单表
 * Created by playboy on 2017/6/5.
 */
@TableName("order_form")
public class OrderForm extends DataEntity<OrderForm>{

    @TableField(exist = false)
    private static final long serialVersionUID = 1L;

    /**
     * 订单编号
     */
    @NotBlank(message = "订单编号不能为空")
    @TableField(value = "order_number")
    private String  orderNumber;

    /**
     * 平台标识
     */
    @NotBlank(message = "平台来源不能为空")
    @TableField(value = "sign")
    private String  sign;

    /**
     * 购买时间
     */
    @NotNull(message = "购买时间不能为空")
    @TableField(value = "order_date")
    private Date orderDate;

    /**
     * 缴费方式(1:一次性,2:分期)
     */
    @NotBlank(message = "缴费方式不能为空")
    @TableField(value = "payment_method")
    private String paymentMethod;


    /**
     * 分期期数
     */
    @TableField(value = "stages_number")
    private String stagesNumber;
    /**
     * 订单有效期
     */
    @NotNull(message = "订单有效期不能为空")
    @TableField(value = "start_date")
    private Date startDate;

    /**
     * 订单有效期
     */
    @NotNull(message = "订单有效期不能为空")
    @TableField(value = "end_date")
    private Date endDate;
    /**
     * 发票类型
     */
    @TableField(value = "invoice_type")
    private String invoiceType;
    /*
    *
    * 踏勘步骤
    * */
    @TableField(value = "step")
    private String step;
    /**
     * 项目名称ID
     */
    @TableField(value = "project_name")
    private String projectName;
    /**
     * 项目ID(风评活动ID)
     */
    @TableField(value = "risk_activity_id")
    private String riskActivityID;

    /**
     * 产品id
     */
    @NotNull(message = "产品不能为空")
    @TableField(value = "product_id")
    private Long productId;



    @TableField(exist = false)
    private String productType;//暂时用于pss分类获取

    /**
     * 用户ID 购买用户
     * @return
     */
    @NotBlank(message = "购买用户不能为空")
    @TableField(value = "web_user_id")
    private String webUserId;

    /**
     * 币种
     */
    @NotBlank(message = "币种不能为空")
    @TableField(value = "currency")
    private String currency;

    /**
     * 省
     */
    @NotBlank(message = "地区省不能为空")
    @TableField(value="province")
    private String province;

    /**
     * 市
     */
    @NotBlank(message = "地区市不能为空")
    @TableField(value="city")
    private String city;

    /**
     * 区/县
     */
    @NotBlank(message = "地区/县不能为空")
    @TableField(value="district")
    private String district;

    /**
     * 详细地址
     */
    @TableField(value="address")
    private String address;

    /**
     * 所属分销商
     */
    @TableField(value="distributor_id")
    private String distributorId;

    /**
     * 所属资源方
     */
    @TableField(value="channel_id")
    private Long channelId;

    /**
     * 订单金额
     */
    @NotNull(message = "订单金额不能为空")
    @TableField(value="fcy")
    private BigDecimal fcy;

    /**
     * 第三方平台订单ID
     */
    @NotBlank(message = "关联平台ID不能为空")
    @TableField(value="web_order_form_id")
    private String webOrderFormId;

    /**
     * 保单号
     */
    @TableField(value="policy_no")
    private String policyNo;

    /**
     * 保全类型(如果是保全类订单时有值)
     */
    @TableField(value="revise_type")
    private String reviseType;

    /**
     * 保全操作人员数量
     */
    @TableField(value="revise_num")
    private String reviseNum;

    /**
     * 父订单ID
     */
    @TableField(value="parent_order_form_id")
    private String parentOrderFormId;

    public String getWebOrderFormId() {
        return webOrderFormId;
    }

    public void setWebOrderFormId(String webOrderFormId) {
        this.webOrderFormId = webOrderFormId;
    }

    private transient OrderPayment orderPayment;

    private transient SysCompany distributor;

    private transient SysCompany channel;

    public OrderPayment getOrderPayment() {
        return orderPayment;
    }

    public void setOrderPayment(OrderPayment orderPayment) {
        this.orderPayment = orderPayment;
    }

    public String getDistributorId() {
        return distributorId;
    }

    public void setDistributorId(String distributorId) {
        this.distributorId = distributorId;
    }

    public Long getChannelId() {
        return channelId;
    }

    public void setChannelId(Long channelId) {
        this.channelId = channelId;
    }

    public String getStep() {
        return step;
    }

    public void setStep(String step) {
        this.step = step;
    }

    public OrderForm(Long id){
        super(id);
    }
    public OrderForm(){
    }


    public String getProjectName() { return projectName; }

    public void setProjectName(String projectName) { this.projectName = projectName; }

    public String getRiskActivityID() { return riskActivityID;
    }

    public void setRiskActivityID(String riskActivityID) { this.riskActivityID = riskActivityID; }

    public BigDecimal getFcy() {
        return fcy;
    }

    public void setFcy(BigDecimal fcy) {
        this.fcy = fcy;
    }

    public SysCompany getDistributor() {
        return distributor;
    }

    public void setDistributor(SysCompany distributor) {
        this.distributor = distributor;
    }

    public SysCompany getChannel() {
        return channel;
    }

    public void setChannel(SysCompany channel) {
        this.channel = channel;
    }

    public String getStagesNumber() {
        return stagesNumber;
    }

    public void setStagesNumber(String stagesNumber) {
        this.stagesNumber = stagesNumber;
    }

    public String getProvince() {
        return province;
    }

    public void setProvince(String province) {
        this.province = province;
    }

    public String getCity() {
        return city;
    }

    public void setCity(String city) {
        this.city = city;
    }

    public String getDistrict() {
        return district;
    }

    public void setDistrict(String district) {
        this.district = district;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getWebUserId() {
        return webUserId;
    }

    public void setWebUserId(String webUserId) {
        this.webUserId = webUserId;
    }

    public String getCurrency() {
        return currency;
    }

    public void setCurrency(String currency) {
        this.currency = currency;
    }

    public String getOrderNumber() {
        return orderNumber;
    }

    public void setOrderNumber(String orderNumber) {
        this.orderNumber = orderNumber;
    }

    public String getSign() {
        return sign;
    }

    public void setSign(String sign) {
        this.sign = sign;
    }

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    public Date getOrderDate() {
        return orderDate;
    }

    public void setOrderDate(Date orderDate) {
        this.orderDate = orderDate;
    }

    public String getPaymentMethod() {
        return paymentMethod;
    }

    public void setPaymentMethod(String paymentMethod) {
        this.paymentMethod = paymentMethod;
    }

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getInvoiceType() {
        return invoiceType;
    }

    public void setInvoiceType(String invoiceType) {
        this.invoiceType = invoiceType;
    }

    public Long getProductId() {
        return productId;
    }

    public void setProductId(Long productId) {
        this.productId = productId;
    }


    public String getProductType() {
        return productType;
    }

    public void setProductType(String productType) {
        this.productType = productType;
    }
    public String getPolicyNo() {
        return policyNo;
    }

    public void setPolicyNo(String policyNo) {
        this.policyNo = policyNo;
    }

    public String getReviseType() {
        return reviseType;
    }

    public void setReviseType(String reviseType) {
        this.reviseType = reviseType;
    }

    public String getReviseNum() {
        return reviseNum;
    }

    public void setReviseNum(String reviseNum) {
        this.reviseNum = reviseNum;
    }

    public String getParentOrderFormId() {
        return parentOrderFormId;
    }

    public void setParentOrderFormId(String parentOrderFormId) {
        this.parentOrderFormId = parentOrderFormId;
    }
}

