package com.bcxin.models.order.service.impl;

import com.bcxin.core.base.service.impl.BaseServiceImpl;
import com.bcxin.core.dao.SysAreaRelationDao;
import com.bcxin.core.entity.SysAreaRelation;
import com.bcxin.core.entity.SysCompany;
import com.bcxin.core.service.SysCompanyService;
import com.bcxin.models.order.dao.OrderFormDao;
import com.bcxin.models.order.dao.OrderPaymentDao;
import com.bcxin.models.order.dto.OrderCompanyRelationDto;
import com.bcxin.models.order.entity.OrderCompanyRelation;
import com.bcxin.models.order.entity.OrderForm;
import com.bcxin.models.order.entity.OrderPayment;
import com.bcxin.models.order.service.OrderCompanyRelationService;
import com.bcxin.models.order.service.OrderFormService;
import com.bcxin.models.product.dao.ProductInsDao;
import com.bcxin.models.product.dao.ProductRelationDao;
import com.bcxin.models.product.dto.ProductRelationDto;
import com.bcxin.models.product.service.ProductRelationService;
import com.bcxin.models.product.util.SendEmailAndMsgUtil;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.mybatisplus.toolkit.StringUtils;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 订单管理
 * Created by playboy on 2017/6/7.
 */
@Service
@Transactional
public class OrderFormServiceImpl extends BaseServiceImpl<OrderFormDao,OrderForm> implements OrderFormService {


    private ProductInsDao productInsDao;
    /**
     * 限制销售区域服务
     */
    @Autowired
    private SysAreaRelationDao sysAreaRelationDao;

    @Autowired
    private OrderPaymentDao orderPaymentDao;

    @Autowired
    private SysCompanyService sysCompanyService;

    @Autowired
    private OrderCompanyRelationService orderCompanyRelationService;

    @Autowired
    private ProductRelationService productRelationService;

    @Autowired
    private ProductRelationDao productRelationDao;

    @Override
    public Map<String, Object> get(OrderForm orderForm) {
        return dao.get(orderForm);
    }

    @Override
    public List<Map<String, Object>> findList(OrderForm orderForm) {
        return null;
    }

    @Override
    public Page<Map<String,Object>> findPage(Page<Map<String, Object>> page, OrderForm orderForm) {
        this.dataScopeFilter(orderForm, "b.supplier_id");
        List<Map<String, Object>> list = dao.findList(page, orderForm);
        //设置供应商、资源方、一级分销商、二级分销商、三级分销商
        list = list.stream().filter(s -> {
            //分销和渠道商处理
            List<OrderCompanyRelationDto> list1 = orderCompanyRelationService.findByOrderIdList(Long.valueOf((String) s.get("id")));
            //资源方(渠道商)
            List channelList = list1.stream().filter(distributor -> distributor.getType().equals(SysCompany.COMPANY_TYPE_CHANNEL)).map(OrderCompanyRelationDto::getCompanyName).collect(Collectors.toList());
            //所有分销商
            List<OrderCompanyRelationDto> distributorList = list1.stream().filter(branch -> branch.getType().equals(SysCompany.COMPANY_TYPE_DISTRIBUTOR)).collect(Collectors.toList());
            List firstList = distributorList.stream().filter(first -> SysCompany.DISTRIBUTIONLEVEL_ONE.equalsIgnoreCase(first.getDistributionLevel())).map(OrderCompanyRelationDto::getCompanyName).collect(Collectors.toList());
            List secondList = distributorList.stream().filter(first -> SysCompany.DISTRIBUTIONLEVEL_TWO.equalsIgnoreCase(first.getDistributionLevel())).map(OrderCompanyRelationDto::getCompanyName).collect(Collectors.toList());
            List thirdList = distributorList.stream().filter(first -> SysCompany.DISTRIBUTIONLEVEL_THREE.equalsIgnoreCase(first.getDistributionLevel())).map(OrderCompanyRelationDto::getCompanyName).collect(Collectors.toList());
            //分销商还要分三级
            //list转string，用逗号隔开
            String channelStr = String.join("<br>", channelList);
            String firstStr = String.join("", firstList);
            String secondStr = String.join("", secondList);
            String thirdStr = String.join("", thirdList);
            s.put("channel", channelStr);
            s.put("distributorFirst", firstStr);
            s.put("distributorSecond", secondStr);
            s.put("distributorThird", thirdStr);

            //如果是保险
            if ("6".equalsIgnoreCase(String.valueOf(s.get("productType")))) {
                //设置供应商 商品可以直接推导出供应商,从产品表直接获取
                ProductRelationDto productRelationDto=new ProductRelationDto();
                productRelationDto.setProductId((Long)s.get("productId"));
                productRelationDto.setType(4);//这里类型有点奇怪，4是供应商
                List<ProductRelationDto> prList=productRelationDao.getSupplyName(productRelationDto);
                //供应商
                List supplyList = prList.stream().map(ProductRelationDto::getSubsidiaryName).collect(Collectors.toList());
                String supplyStr = String.join("<br>", supplyList);
                s.put("supply", supplyStr);
            }
            //如果是PSS，已经设置了供应商，可能有多个
            if ("7".equalsIgnoreCase(String.valueOf(s.get("productType")))) {
                //用订单号在公司订单表，查出找机构，拼接成供应商字段
                //供应商
                List supplyList = list1.stream().filter(suplly -> suplly.getType().equals(SysCompany.COMPANY_TYPE_SUPPLIER)).map(OrderCompanyRelationDto::getCompanyName).collect(Collectors.toList());
                String supplyStr = String.join("<br>", supplyList);
                s.put("supply", supplyStr);
            }
            return true;
        }).collect(Collectors.toList());
        page.setRecords(list);
        return page;
    }

    /**
     *
     * 数据验证
     * @return
     */
    @Override
    public boolean checkData(OrderForm orderForm,StringBuffer sb) {
        return false;
    }

    @Override
    public void saveOrderFormOrPayment(OrderForm orderForm, OrderPayment orderPayment) {
        if(orderForm!=null) {
            if(StrUtil.isBlank(orderForm.getWebUserId())){
                logger.error("订单同步失败,购买人为空!");
                return;
            }
            super.save(orderForm);
            if("PSS".equals(orderForm.getSign())){
                //设置分销商
                setDistributorPss(orderForm);
                //设置供应商(风评供应商)
                setSupplyIdPss(orderForm);
            }else{
                //设置分销商  存在多个分销商核心去处理 todo 核心哪里处理了？
                setDistributorId(orderForm);
            }
            //设置资源方
            setChannelId(orderForm);
            if(orderPayment!=null) {
                orderPayment.setOrderFormId(orderForm.getId());
            }
        }
        int count = orderPaymentDao.selectCount(new EntityWrapper().addFilter("order_form_id={0} and pay_serial_number={1}",orderPayment.getOrderFormId(),orderPayment.getPaySerialNumber()));
        if(count>0){
            orderPaymentDao.update(orderPayment,new EntityWrapper().addFilter("order_form_id={0} and pay_serial_number={1}",orderPayment.getOrderFormId(),orderPayment.getPaySerialNumber()));
        }else {
            orderPaymentDao.insert(orderPayment);
        }
    }

    private void setSupplyIdPss(OrderForm orderForm) {

        //根据产品获取 所有供应商
        //根据省市区 筛选 对应的供应商
        //可以多个
        Map<String,Object> map= Maps.newHashMap();
        map.put("province",orderForm.getProvince());
        map.put("city",orderForm.getCity());
        map.put("area",orderForm.getDistrict());
        map.put("productId", orderForm.getProductId());
        List<SysCompany> list= sysCompanyService.findSupplyListByArea(map);
        if(list==null){
            logger.info("订单未找到归属的供应商:订单号："+orderForm.getOrderNumber());
        }
        else{
            logger.info("订单找到归属的供应商个数"+list.size());
            saveOrderCompanyRelationSupply(orderForm,list);

        }


    }

    private void saveOrderCompanyRelationSupply(OrderForm orderForm, List<SysCompany> list) {
        List<OrderCompanyRelation> relationList = Lists.newArrayList();
        for(SysCompany company:list){
            OrderCompanyRelation orderCompanyRelation = new OrderCompanyRelation();
            orderCompanyRelation.setSysCompany(company);
            orderCompanyRelation.setOrderForm(orderForm);
            orderCompanyRelation.setType(SysCompany.COMPANY_TYPE_SUPPLIER);
            relationList.add(orderCompanyRelation);
        }
        if(relationList.size()>0){
            orderCompanyRelationService.insertBatch(relationList);
        }


    }

    private void setDistributorPss(OrderForm orderForm) {

        //获取展会类型
        String  isExhibition = getExhibition(orderForm.getProductType());

        //根据产品获取分销商 ，在根据省市区 查询所有分销商

        Map<String,Object> map= Maps.newHashMap();
        map.put("province",orderForm.getProvince());
        map.put("city",orderForm.getCity());
        map.put("area",orderForm.getDistrict());
        map.put("isExhibition",isExhibition);
        map.put("productId", orderForm.getProductId());
        List<SysCompany> list= sysCompanyService.findListByArea(map);
        //再根据产品类型(加风评类型加展会类型)区分出 单独分销商
        if(list==null){
            logger.info("订单未找到归属的分销商:订单号："+orderForm.getOrderNumber());
        }
        else if(list.size()==1){
            //归属
            orderForm.setDistributorId(StrUtil.obj2Str(list.get(0).getId()));
            saveOrderCompanyRelation(orderForm);
        }else{
            //如果存在多个 分销商ID 取 最后一层极的
            String distributorId="";
            int  maxNum =0;
            for (SysCompany c:list){
                if(maxNum<Integer.parseInt(c.getDistributionLevel())){
                    maxNum=Integer.parseInt(c.getDistributionLevel());
                    distributorId=c.getId()+"";
                }
                logger.error("订单号"+orderForm.getOrderNumber()+"出现多个归属商"+c.getCompanyName()+c.getId());
            }
            orderForm.setDistributorId(distributorId);
            saveOrderCompanyRelation(orderForm);
        }


    }
    private void setExportPss(OrderForm orderForm) {

        String distributorId= dao.findCompanyIDByWebUserID(orderForm.getWebUserId());
        if ("null".equals(distributorId)||"".equals(distributorId)){
            SendEmailAndMsgUtil.sendEmail("订单未找到归属的分销商","订单未找到归属的分销商,---表Order form :订单号（id）:"+orderForm.getId()+",这个专家没有升级成分销商（web_user_id）："+orderForm.getWebUserId()+"。操作：将专家升级成分销商后，将相对应的sys_company的字段relate_to_terminal复制到表order_company_relation","87751510@qq.com");
            logger.info("订单未找到归属的分销商");
        }
        orderForm.setDistributorId(distributorId);
        OrderCompanyRelation orderCompanyRelation = new OrderCompanyRelation();
        SysCompany sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("id={0}",orderForm.getDistributorId()));
        orderCompanyRelation.setSysCompany(sysCompany);
        orderCompanyRelation.setOrderForm(orderForm);
        orderCompanyRelation.setType(SysCompany.COMPANY_TYPE_DISTRIBUTOR);
        orderCompanyRelationService.save(orderCompanyRelation);

    }
    private String getExhibition(String productType) {

        if(productType!=null){
            //0,全部， 1 是展会 ，2 不是展会
            if("3".equals(productType)){
                return "1";
            }else{
                return "2";
            }

//            1  体育比赛活动
//            2  演唱会、音乐会等文艺演出活动
//            3  展览会、展销会、交流会等活动
//            4  游园、灯会、庙会、花会、焰火晚会、美食节等活动
//            5  招聘会、推介会、现场开奖的彩票销售等活动
//            6  公益慈善活动
//            7  其他类活动
        }

        return  null;

    }

    /**
     * 设置分销商
     * 先查找应用平台归属的分销商,后查找购买人是否分销商.
     * 分销商归属:1:验证购买用户是否分销商.2,验证分享标识(
     * * @param orderForm
     * @return
     */
    private void setDistributorId(OrderForm orderForm){


        String share = orderForm.getDistributorId();
        //验证分享标识
        if(StrUtil.isNotBlank(share)){
            SysCompany sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("share_identifier={0}",share));
            if(null!=sysCompany&&sysCompany.getId()!=null){
                orderForm.setDistributorId(StrUtil.obj2Str(sysCompany.getId()));
            }
        }
        //判断推荐码是否存在
        if(StrUtil.isNotBlank(orderForm.getDistributorId())){
            //存在推荐码以推荐码为准
            SysCompany sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("id={0}",orderForm.getDistributorId()));
            if(null!=sysCompany&&sysCompany.getId()!=null){
                orderForm.setDistributorId(StrUtil.obj2Str(sysCompany.getId()));
            }
            //验证推荐码是否是终端用户提升的分销商对应的业务支撑id.
            sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("relate_to_terminal={0}",orderForm.getDistributorId()));
            if(null!=sysCompany&&sysCompany.getId()!=null){
                orderForm.setDistributorId(StrUtil.obj2Str(sysCompany.getId()));
            }
            if(StringUtils.isEmpty(orderForm.getDistributorId())){
                //验证是否终端企业提升的分销商
                sysCompany = sysCompanyService.getSysCompanyByUserId(StrUtil.toLong(orderForm.getDistributorId()));
                if(null!=sysCompany&&sysCompany.getId()!=null) {
                    sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("relate_to_terminal={0}", sysCompany.getId()));
                    if(null!=sysCompany&&sysCompany.getId()!=null){
                        orderForm.setDistributorId(StrUtil.obj2Str(sysCompany.getId()));
                    }
                }
            }

        }else{
            //验证是否是终端用户提升的分销商.
            if(StrUtil.isNotBlank(orderForm.getWebUserId())){
                SysCompany sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("relate_to_terminal={0}",orderForm.getWebUserId()));
                if(null!=sysCompany&&sysCompany.getId()!=null){
                    orderForm.setDistributorId(StrUtil.obj2Str(sysCompany.getId()));
                }else {
                    //验证是否终端企业提升的分销商
                    sysCompany = sysCompanyService.getSysCompanyByUserId(StrUtil.toLong(orderForm.getWebUserId()));
                    if(null!=sysCompany&&sysCompany.getId()!=null) {
                        sysCompany = sysCompanyService.selectOne(new EntityWrapper().addFilter("relate_to_terminal={0}", sysCompany.getId()));
                        if(null!=sysCompany&&sysCompany.getId()!=null){
                            orderForm.setDistributorId(StrUtil.obj2Str(sysCompany.getId()));
                        }
                    }
                }
            }
        }

        if(StrUtil.isBlank(orderForm.getDistributorId())){
            logger.info("分销商获取失败:"+orderForm.getOrderNumber());
            return;
        }

        //向上查找,如果当前无区域配置信息,查询上级分销商的区域,直到查询1级分销商所设置区域
        List<SysAreaRelation> list = sysAreaRelationDao.findList(new SysAreaRelation(new SysCompany(StrUtil.toLong(orderForm.getDistributorId()))));
        if(null==list||list.size()==0){
            SysCompany sysCompany = sysCompanyService.get(StrUtil.toLong(orderForm.getDistributorId()));
            if(null!=sysCompany) {
                list = sysAreaRelationDao.findList(new SysAreaRelation(new SysCompany(sysCompany.getParent().getId())));
                if(null==list||list.size()==0){
                     sysCompany = sysCompanyService.get(sysCompany.getParent().getId());
                     if(null!=sysCompany) {
                         list = sysAreaRelationDao.findList(new SysAreaRelation(new SysCompany(sysCompany.getParent().getId())));
                     }
                }
            }
        }
        Long companyId = this.getcheckSysRegionCompanyId(orderForm,list);
        if(orderForm.getDistributorId().equals(StrUtil.obj2Str(companyId))){
            orderForm.setDistributorId(StrUtil.obj2Str(companyId));
        }else {
            orderForm.setDistributorId(null);
        }
        saveOrderCompanyRelation(orderForm);
    }

    /**
     * 保存订单与分销商关联
     */
    private void saveOrderCompanyRelation(OrderForm orderForm){
        if(null!=orderForm.getDistributorId()){
            SysCompany sysCompany = sysCompanyService.get(StrUtil.toLong(orderForm.getDistributorId()));
            if(null==sysCompany){
                return ;
            }
            //1级分销商不需要关联上级
            if(SysCompany.DISTRIBUTIONLEVEL_ONE.equals(sysCompany.getDistributionLevel())){
                saveRelation(Lists.newArrayList(sysCompany),orderForm);
            }else if(SysCompany.DISTRIBUTIONLEVEL_TWO.equals(sysCompany.getDistributionLevel())){
                List<SysCompany> list = sysCompanyService.selectList(new EntityWrapper().addFilter("id={0} or id={1}",sysCompany.getId(),sysCompany.getParentId(),sysCompany.getId()));
                saveRelation(list,orderForm);
            }else if(SysCompany.DISTRIBUTIONLEVEL_THREE.equals(sysCompany.getDistributionLevel())){
                List<SysCompany> list = sysCompanyService.selectList(new EntityWrapper().addFilter("instr({0},id)>0 or id={1}",sysCompany.getParentIds(),sysCompany.getId()));
                saveRelation(list,orderForm);
            }
        }
    }

    /**
     * 保存关联
     */
    private void saveRelation(List<SysCompany> list,OrderForm orderForm){
        List<OrderCompanyRelation> relationList = Lists.newArrayList();
        Map<String,Object> params = Maps.newHashMap();
        params.put("productId",orderForm.getProductId());
        params.put("type","1");
        params.put("companyType","2");
        //获取产品下的所有分销商关联
        List<Long> companyListIdList = productRelationService.getCompanyListId(params);
        if(null!=list&&list.size()>0){
            for (SysCompany company:list) {
                if(companyListIdList.contains(company.getId())) {
                    OrderCompanyRelation orderCompanyRelation = new OrderCompanyRelation();
                    orderCompanyRelation.setSysCompany(company);
                    orderCompanyRelation.setOrderForm(orderForm);
                    orderCompanyRelation.setType(SysCompany.COMPANY_TYPE_DISTRIBUTOR);
                    relationList.add(orderCompanyRelation);
                }
            }
        }
        if(relationList.size()>0){
            orderCompanyRelationService.insertBatch(relationList);
        }
    }

    /**
     * 设置资源方(产品确认同一个地区不会超过20个,上线20个)
     * @param orderForm
     */
    private void setChannelId(OrderForm orderForm){
        Map<String,Object> params = Maps.newHashMap();
        params.put("productId",orderForm.getProductId());
        params.put("type","2");
        params.put("companyType","4");
        List<Long> companyListIdList = productRelationService.getCompanyListId(params);
        List<OrderCompanyRelation> relationList = Lists.newArrayList();
        for(Long companyId:companyListIdList) {
            SysAreaRelation sysAreaRelation = new SysAreaRelation();
            sysAreaRelation.setType(SysAreaRelation.CHANNEL_TYPE);
            sysAreaRelation.setSysCompany(new SysCompany(companyId));
            List<SysAreaRelation> list = sysAreaRelationDao.findList(sysAreaRelation);
            Long id= getcheckSysRegionCompanyId(orderForm, list);
            if(id.equals(companyId)) {
                OrderCompanyRelation orderCompanyRelation = new OrderCompanyRelation();
                orderCompanyRelation.setSysCompany(new SysCompany(id));
                orderCompanyRelation.setOrderForm(orderForm);
                orderCompanyRelation.setType(SysCompany.COMPANY_TYPE_CHANNEL);
                relationList.add(orderCompanyRelation);
            }
        }
        if(relationList.size()>0){
            orderCompanyRelationService.insertBatch(relationList);
        }
    }

    /**
     * 比对地区 返回分销商或者资源方
     * @return
     */
    private Long getcheckSysRegionCompanyId(OrderForm orderForm,List<SysAreaRelation> list){
        Long companyId = 0L;
        if(null!=list&&list.size()>0){
            for(SysAreaRelation sysAreaRelation:list){
                companyId = sysAreaRelation.getSysCompany().getId();
                StringBuffer stringBuffer = new StringBuffer();
                StringBuffer orderBuffer = new StringBuffer();
                if(!sysAreaRelation.getCity().getId().equals(0L)){
                    stringBuffer.append(sysAreaRelation.getCity().getId()).append(",");
                    orderBuffer.append(orderForm.getDistrict().trim()).append(",");
                }
                if(!sysAreaRelation.getCitytype().getId().equals(0L)){
                    stringBuffer.append(sysAreaRelation.getCitytype().getId()).append(",");
                    orderBuffer.append(orderForm.getCity().trim()).append(",");
                }
                if(!sysAreaRelation.getProvince().getId().equals(0L)){
                    stringBuffer.append(sysAreaRelation.getProvince().getId()).append(",");
                    orderBuffer.append(orderForm.getProvince().trim()).append(",");
                }
                if(stringBuffer.toString().equals(orderBuffer.toString())){
                    break;
                }
                companyId=0L;
            }
        }
        return companyId;
    }

    /**
     * 核心系统同步专用
     * @param parmas
     * @return
     */
    @Override
    public List<Map<String,Object>> selectToHxxt(Map<String,Object> parmas){
        return dao.selectToHxxt(parmas);
    }
    @Override
    public List<Map<String, Object>> findExport(){
        return dao.findExport();
    }
    @Override
    public int insertExportMessage(Long companyID,String realName,String subsidiaryName){
        return dao.insertExportMessage(companyID,realName,subsidiaryName);
    }
    @Override
    public int selectByRiskActivityIDAndProductID(Long risk_activity_id,Long product_id,Long export_id,String step){

        return dao.selectByRiskActivityIDAndProductID(risk_activity_id,product_id,export_id,step);
    }

    @Override
    public boolean updataExportOrderFrom( Long risk_activity_id, Long export_id,
                                   Long product_id, String project_name,String step){

        OrderForm orderForm2= selectOne(new EntityWrapper().addFilter("web_order_form_id={0}",risk_activity_id));
        if (!("null".equals(orderForm2.getId())||"".equals(orderForm2.getId()))){
            OrderForm orderForm = new OrderForm();
            orderForm.preInsert();
            orderForm.setSign("PSS");//标志位是pss
            orderForm.setProductId(product_id);//产品名称
            orderForm.setRiskActivityID(String.valueOf(risk_activity_id));//风评活动id
            orderForm.setWebUserId(String.valueOf(export_id));//订单的专家id
            orderForm.setProjectName(project_name);//项目名称
            orderForm.setStep(step);//踏勘步骤
          String orderNumber=  orderForm2.getOrderNumber();
            Date now = new Date();

            int ran= (int) ((Math.random()*9+1)*1000);
            String newOrderNumber=orderNumber+export_id+step+ran;

            orderForm.setOrderNumber(newOrderNumber);
            orderForm.setProvince(orderForm2.getProvince());//省市区
            orderForm.setCity(orderForm2.getCity());
            orderForm.setDistrict(orderForm2.getDistrict());



            orderForm.setCreateTime(now);
            orderForm.setUpdateTime(now);
            orderForm.setOrderDate(now);
            setExportPss(orderForm);
            boolean rs = insertOrUpdate(orderForm);
            return rs;
        }
        return false;
    }
}
