package com.bcxin.models.order.web;

import com.bcxin.core.base.web.BaseController;
import com.bcxin.core.entity.SysUser;
import com.bcxin.core.util.SysUserUtils;
import com.bcxin.models.order.entity.OrderForm;
import com.bcxin.models.order.entity.OrderPayment;
import com.bcxin.models.order.service.OrderFormService;
import com.bcxin.models.order.service.OrderPaymentService;
import com.bcxin.models.platform.ConstantURL;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.Map;

/**
 * 订单中心
 * Created by playboy on 2017/6/7.
 */
@Controller
@RequestMapping("${systemPath}/orderForm")
public class OrderFormController extends BaseController {

    /**
     * 订单
     */
    @Autowired
    private OrderFormService orderFormService;

    /**
     * 交易记录
     */
    @Autowired
    private OrderPaymentService orderPaymentService;

    //list
    private static final String ADD = getViewPath("admin/order/order_form_add");
    //view
    private static final String LIST = getViewPath("admin/order/order_form_list");

    private static final String INSLIST =  getViewPath(ConstantURL.ORDERFORMLISTINS);


    /**
     * 分销商订单列表
     * @param model
     * @return
     */
    @RequiresPermissions("order:form:list")
    @RequestMapping("")
    public String listView(Model model){
        return LIST;
    }

    /**
     * 保险供应商订单列表
     * @return
     */
    @RequiresPermissions("order:form:insList")
    @RequestMapping("insList")
    public String insList(Model model){
        SysUser user = SysUserUtils.getUser();
        if(user.getSysCompany()!=null&&user.getSysCompany().getCompanyType()!=null){
            String url = INSLIST+ SysUserUtils.getUser().getSysCompany().getId();
            return redirectTo(url);
        }else{
            return redirectTo(INSLIST+"sup_admin");
        }
    }

    @RequiresPermissions("order:form:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(Model model,OrderForm orderForm){
        Page<Map<String,Object>> page =this.getPage();
        orderFormService.findPage(page,orderForm);
        return jsonPage(page);
    }



    @RequiresPermissions("order:form:form")
    @RequestMapping("/form")
    public String form(Model model,@RequestParam(required=false)Long id){
        String flag = this.getParameter("modify");
        if(id!=null) {
            Map<String,Object> orderForm = orderFormService.get(new OrderForm(id));
            model.addAttribute("orderForm", orderForm);
        }
        model.addAttribute("modify",flag);
        return ADD;
    }

    /**
     * 获取支付记录
     * @param model
     * @param orderFormId
     * @return
     */
    @RequiresPermissions("order:form:form")
    @RequestMapping("/getOrderPaymentListData")
    @ResponseBody
    public String getOrderPaymentListData(Model model, Long orderFormId){
        Page<OrderPayment> page =this.getPage();
        orderPaymentService.selectPage(page,new EntityWrapper(new OrderPayment(orderFormId)).addFilter("remaining_number<>{0}",0));
        return jsonPage(page);
    }
}
