package com.bcxin.models.product.entity;

import com.bcxin.core.base.entity.DataEntity;
import com.bcxin.core.entity.SysCompany;
import com.bcxin.mybatisplus.annotations.TableField;
import com.bcxin.mybatisplus.annotations.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import org.hibernate.validator.constraints.NotBlank;

import javax.validation.constraints.Max;
import javax.validation.constraints.NotNull;
import java.util.List;


/**
 * 
 * @author 风评产品
 *
 */
@TableName("product_risk")
public class ProductRisk extends DataEntity<ProductRisk> {
	@TableField(exist = false)
	private static final long serialVersionUID = 1L;

	/**
	 * 产品名称
	 */
	@NotBlank(message = "产品名称不能为空")
	@TableField(value = "name")
	private String name;
	/**
	 *所属供应商这个去掉用关联表
	 */
	@TableField(value = "supplier_id",el = "supplier.id")
	private SysCompany supplier;

	/**
	 * 产品类型
	 */
	@NotNull(message = "产品类型不能为空")
	@TableField(value = "product_type")
	private String productType;

	/**
	 * 图片
	 */
	@TableField(value = "picture")
	private String picture;

	/**
	 * 产品简述
	 */
	@TableField(value = "intro")
	private String intro;

	/**
	 * 省
	 */
	@NotBlank(message = "请选择省份")
	@TableField(value="province")
	private String province;

	/**
	 * 市
	 */
	@NotBlank(message = "请选择市")
	@TableField(value="city")
	private String city;

	/**
	 * /区
	 */
	@TableField(value="district")
	private String district;

	/**
	 * 地址
	 */
	@TableField(value="address")
	private String address;

	/**
	 * 场馆名称
	 */
	@NotBlank(message = "场馆名称不能为空")
	@TableField(value="venue_name")
	private String venueName;

	/**
	 * 周边停车场
	 */
	@TableField(value="peripheral_parking")
	private String peripheralParking;

	/**
	 * 容纳人数
	 */
	@Max(value = 999999,message = "容纳人数最大不能超过999999")
	@TableField(value="accommodating_number")
	private Integer accommodatingNumber;

	@Max(value = 9999,message = "排序最大不能超过9999")
	@TableField(value="sort")
	private Integer sort;

	/**
	 * 分类
	 */
	@NotNull(message = "分类不能为空")
	@TableField(value = "responsibility_type",el = "responsibilityType.id")
	private ProductType responsibilityType;



	/**
	 * 供应商个数
	 */
	@TableField(exist = false)
	private  String  supplierCnt;

	public ProductType getResponsibilityType() {
		return responsibilityType;
	}

	public void setResponsibilityType(ProductType responsibilityType) {
		this.responsibilityType = responsibilityType;
	}

	public Integer getSort() {
		return sort;
	}

	public void setSort(Integer sort) {
		this.sort = sort;
	}

	public String getVenueName() {
		return venueName;
	}

	public void setVenueName(String venueName) {
		this.venueName = venueName;
	}

	public String getPeripheralParking() {
		return peripheralParking;
	}

	public void setPeripheralParking(String peripheralParking) {
		this.peripheralParking = peripheralParking;
	}

	public Integer getAccommodatingNumber() {
		return accommodatingNumber;
	}

	public void setAccommodatingNumber(Integer accommodatingNumber) {
		this.accommodatingNumber = accommodatingNumber;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public String getDistrict() {
		return district;
	}

	public void setDistrict(String district) {
		this.district = district;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getProvince() {
		return province;
	}

	public void setProvince(String province) {
		this.province = province;
	}

	public ProductRisk(Long id){
		super(id);
	}
	public ProductRisk(Long id,Integer status){
		super(id);
		super.setStatus(status);
	}

	public ProductRisk(){ }

	public String getProductType() {
		return productType;
	}

	public void setProductType(String productType) {
		this.productType = productType;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public SysCompany getSupplier() {
		return supplier;
	}

	public void setSupplier(SysCompany supplier) {
		this.supplier = supplier;
	}

	public String getPicture() {
		return picture;
	}

	public void setPicture(String picture) {
		this.picture = picture;
	}

	public String getIntro() {
		return intro;
	}

	public void setIntro(String intro) {
		this.intro = intro;
	}

	public String getSupplierCnt() {
		return supplierCnt;
	}

	public void setSupplierCnt(String supplierCnt) {
		this.supplierCnt = supplierCnt;
	}

	@JsonIgnore
	public static void sortList(List<ProductType> list, List<ProductType> sourcelist, Long id){
		for (int i=0; i<sourcelist.size(); i++){
			ProductType e = sourcelist.get(i);
			if (e.getId().equals(id)){
				list.add(e);
				//判断是否有父节点
				if(e.getParent()!=null && e.getParent().getId()!=null) {
					for (int j = 0; j < sourcelist.size(); j++) {
						ProductType parent = sourcelist.get(j);
						if (e.getParent() != null && e.getParent().getId() != null
								&& parent.getId().equals(e.getParent().getId())) {
							sortList(list, sourcelist,parent.getId());
							break;
						}
					}
				}
			}
		}
	}
}
