package com.bcxin.models.product.entity;

import com.bcxin.core.base.entity.DataEntity;
import com.bcxin.mybatisplus.annotations.TableField;
import com.bcxin.mybatisplus.annotations.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import org.hibernate.validator.constraints.NotBlank;

import java.util.List;

/**
 *
 * 产品类型
 * Created by playboy on 2017/5/27.
 */
@TableName(value = "product_type")
public class ProductType extends DataEntity<ProductType> {

    @TableField(exist = false)
    private static final long serialVersionUID = 1L;

    /**
     * 类型 大类
     */
    @TableField(value = "type")
    @NotBlank(message = "类型不能为空")
    private String type;


    /**
     *  类型名称
     */
    @NotBlank(message = "类型名称不能为空")
    @TableField(value = "type_name")
    private String typeName;

    /**
     * 分类类型(保险分类类型暂时有二种:1:保险责任分类,2:市场属性分类)
     */
    @TableField(value = "ins_type")
    private Integer insType;

    /**
     * 类型等级
     */
    @TableField(value = "type_level")
    private String typeLevel;

    /**
     * 父级ID
     */
    @TableField(value = "parent_id",el = "parent.id")
    private ProductType parent;

    /**
     * 路径
     */
    @TableField(value = "parent_ids")
    private String parentIds;

    /**
     * 排序
     */
    @TableField(value = "sort")
    private Integer sort;

    private transient String startDate;

    public String getStartDate() {
        return startDate;
    }

    public void setStartDate(String startDate) {
        this.startDate = startDate;
    }

    public Integer getInsType() {
        return insType;
    }

    public void setInsType(Integer insType) {
        this.insType = insType;
    }

    public  ProductType(Long id){
        super(id);
    }

    public  ProductType(){

    }
    public ProductType getParent() {
        return parent;
    }

    public void setParent(ProductType parent) {
        this.parent = parent;
    }

    public Integer getSort() {
        return sort;
    }

    public void setSort(Integer sort) {
        this.sort = sort;
    }

    public Long getParentId() {
        Long id = null;
        if (parent != null){
            id = parent.getId();
        }
        return id==null ? 0L : id;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getTypeName() {
        return typeName;
    }

    public void setTypeName(String typeName) {
        this.typeName = typeName;
    }

    public String getTypeLevel() {
        return typeLevel;
    }

    public void setTypeLevel(String typeLevel) {
        this.typeLevel = typeLevel;
    }

    public String getParentIds() {
        return parentIds;
    }

    public void setParentIds(String parentIds) {
        this.parentIds = parentIds;
    }

    @JsonIgnore
    public static void sortList(List<ProductType> list, List<ProductType> sourcelist, Long parentId, boolean cascade){
        for (int i=0; i<sourcelist.size(); i++){
            ProductType e = sourcelist.get(i);
            if (e.getParent()!=null && e.getParent().getId()!=null
                    && e.getParent().getId().equals(parentId)){
                list.add(e);
                if (cascade){
                    // 判断是否还有子节点, 有则继续获取子节点
                    for (int j=0; j<sourcelist.size(); j++){
                        ProductType child = sourcelist.get(j);
                        if (child.getParent()!=null && child.getParent().getId()!=null
                                && child.getParent().getId().equals(e.getId())){
                            sortList(list, sourcelist, e.getId(), true);
                            break;
                        }
                    }
                }
            }
        }
    }
    @JsonIgnore
    public static void sortList(List<ProductType> list, List<ProductType> sourcelist, Long id){
        for (int i=0; i<sourcelist.size(); i++){
            ProductType e = sourcelist.get(i);
            if (e.getId().equals(id)){
                list.add(e);
                //判断是否有父节点
                if(e.getParent()!=null && e.getParent().getId()!=null) {
                    for (int j = 0; j < sourcelist.size(); j++) {
                        ProductType parent = sourcelist.get(j);
                        if (e.getParent() != null && e.getParent().getId() != null
                                && parent.getId().equals(e.getParent().getId())) {
                            sortList(list, sourcelist,parent.getId());
                            break;
                        }
                    }
                }
            }
        }
    }
}
