package com.bcxin.models.product.service.impl;

import com.bcxin.core.base.service.impl.BaseServiceImpl;
import com.bcxin.models.agreement.dao.AgreementDao;
import com.bcxin.models.agreement.dao.ProductRelationFeerateDao;
import com.bcxin.models.agreement.entity.Agreement;
import com.bcxin.models.agreement.entity.AgreementFee;
import com.bcxin.models.agreement.entity.ProductRelationFeerate;
import com.bcxin.models.agreement.entity.ProductRelationFeerateKey;
import com.bcxin.models.agreement.service.AgreementFeeService;
import com.bcxin.models.product.dao.ProductRelationDao;
import com.bcxin.models.product.entity.ProductRelation;
import com.bcxin.models.product.service.ProductRelationService;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.toolkit.IdWorker;
import com.bcxin.util.toolbox.DateUtil;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * 产品关联
 * Created by playboy on 2017/6/4.
 */
@Service
@Transactional
public class ProductRelationServiceImpl extends BaseServiceImpl<ProductRelationDao, ProductRelation> implements ProductRelationService {

    @Autowired
    private ProductRelationFeerateDao productRelationFeerateDao;
    @Autowired
    private AgreementFeeService agreementFeeService;
    @Autowired
    private AgreementDao agreementDao;

    /**
     * 数据验证,验证是否已经存在,存在就不做变更
     *
     * @param productRelation
     * @param msg
     * @return
     */
    @Override
    public boolean checkData(ProductRelation productRelation, String msg) {
        List<ProductRelation> list = dao.selectList(new EntityWrapper<ProductRelation>()
                .addFilter("product_id={0} and relation_id={1} and type={2}"
                        , productRelation.getProductId(), productRelation.getRelationId(), productRelation.getType()));
        if (list.size() > 0) {
            return false;
        }
        return true;
    }

    @Override
    public void checkSave(String productIds, String relationIds, Integer type) {
        List<ProductRelation> list = Lists.newArrayList();
        List<AgreementFee> fees = Lists.newArrayList();
        if (StrUtil.isNotBlank(relationIds) && StrUtil.isNotBlank(productIds) && null != type) {
            for (String id : StrUtil.split(relationIds, ",")) {
                //先查出协议id
                Agreement agreementQuery = new Agreement();
                agreementQuery.setCompanyId(Long.valueOf(relationIds));
                Agreement agreement = agreementDao.selectOne(agreementQuery);
                Long agreementId = null;
                if (agreement != null) {
                    agreementId = agreement.getId();
                }
                for (String pId : StrUtil.split(productIds, ",")) {
                    ProductRelation productRelation = new ProductRelation();
                    productRelation.setRelationId(StrUtil.toLong(id));
                    productRelation.setType(type);
                    productRelation.setProductId(StrUtil.toLong(pId));
                    if (agreementId != null) {
                        AgreementFee agreementFee = new AgreementFee();
                        agreementFee.setAgreementId(agreementId);
                        agreementFee.setCalcMethod("1");
                        agreementFee.setFixedCost(new BigDecimal(0));
                        agreementFee.setFixedAmount(new BigDecimal(0));
                        agreementFee.setTaxRate(new BigDecimal(6));
                        agreementFee.setProductType("6");
                        agreementFee.setTaxScopeStart(new BigDecimal(0));
                        agreementFee.setTaxScopeEnd(new BigDecimal(0));
                        agreementFee.setFeeRate(new BigDecimal(0));
                        agreementFee.setProductId(pId);
                        fees.add(agreementFee);
                    }
                    if (checkData(productRelation, null)) {
                        productRelation.setUpdateTIme(DateUtil.getTimestamp());
                        productRelation.setCreateTime(DateUtil.getTimestamp());
                        list.add(productRelation);
                    }
                }
            }
        }
        if (list.size() > 0) {
            super.insertBatch(list);
        }
        if (fees.size() > 0) {
            agreementFeeService.insertBatch(fees);
        }
    }


    @Override
    public List<Map<String, Object>> getCompanyDList(Map<String, Object> params) {
        return dao.getCompanyDList(params);
    }

    @Override
    public List<Map<String, Object>> getProductListDetails(Map<String, Object> params) {
        return dao.getProductListDetails(params);
    }

    @Override
    public List<Long> getCompanyListId(Map<String, Object> params) {
        return dao.getCompanyListId(params);
    }

    @Override
    public List<Map<String, Object>> selectToHxxt(Map<String, Object> params) {
        return dao.selectToHxxt(params);
    }

    @Override
    public void saveRangeFee(BigDecimal fee, Long productRelationId, Long agreementFeeId, BigDecimal taxScopeStart, BigDecimal taxScopeEnd) {
        //先删除
        ProductRelationFeerateKey key = new ProductRelationFeerateKey();
        key.setAgreementFeeId(agreementFeeId);
        key.setProductRelationId(productRelationId);
        productRelationFeerateDao.deleteByPrimaryKey(key);
        //先插入
        ProductRelationFeerate feerate = new ProductRelationFeerate();
        feerate.setAgreementFeeId(agreementFeeId);
        feerate.setProductRelationId(productRelationId);
        feerate.setFee(fee);
        feerate.setTaxScopeStart(taxScopeStart);
        feerate.setTaxScopeEnd(taxScopeEnd);
        Long id = IdWorker.getId();
        feerate.setId(id);
        productRelationFeerateDao.insertSimple(feerate);
    }
}
