package com.bcxin.models.product.service.impl;

import com.bcxin.core.base.service.impl.BaseServiceImpl;
import com.bcxin.models.product.dao.ProductTypeDao;
import com.bcxin.models.product.entity.ProductType;
import com.bcxin.models.product.service.ProductTypeService;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.util.toolbox.StrUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 *
 * 产品类型
 * Created by playboy on 2017/5/27.
 */
@Service
@Transactional
public class ProductTypeServiceImpl extends BaseServiceImpl<ProductTypeDao,ProductType> implements ProductTypeService {

    @Override
    public void save(ProductType productType) {
        if (productType.getParent() == null || productType.getParentId()==null|| "0".equals(productType.getParentId())){
            productType.setParent(null);
        }else{
            productType.setParent(super.selectById(productType.getParentId()));
        }
        if (productType.getParent() == null){
            productType.setParent(new ProductType(0L));
            productType.getParent().setParentIds(StrUtil.EMPTY);
        }
        // 获取修改前的parentIds，用于更新子节点的parentIds
        String oldParentIds = StrUtil.isBlank(productType.getParentIds())?StrUtil.EMPTY:productType.getParentIds();
        String parentIds = StrUtil.isBlank(productType.getParent().getParentIds())?StrUtil.EMPTY:productType.getParent().getParentIds();
        // 设置新的父节点串
        productType.setParentIds(parentIds+productType.getParent().getId()+",");
        super.save(productType);

        List<ProductType> list = dao.selectList(new EntityWrapper<ProductType>().addFilter("parent_ids like {0}","%,"+productType.getId()+",%"));
        for (ProductType e : list){
            e.setParentIds(e.getParentIds().replace(oldParentIds, productType.getParentIds()));
            dao.updateById(e);
        }
    }

    @Override
    public ProductType get(ProductType productType) {
        return dao.get(productType);
    }

    @Override
    public List<ProductType> findList(ProductType productType) {
        return dao.findList(productType);
    }

    @Override
    public Page<ProductType> findPage(Page<ProductType> page, ProductType productType) {
        page.setRecords(dao.findList(page,productType));
        return page;
    }

    @Override
    public String checkData(ProductType productType) {
        Integer count=0;
        if(productType.getId()==null) {
            count = dao.selectCount(new EntityWrapper<ProductType>().addFilter("type_name={0}", productType.getTypeName()));
        }else {
            count = dao.selectCount(new EntityWrapper<ProductType>().addFilter("type_name={0} and id <>{1}", productType.getTypeName(), productType.getId()));
        }
        if(count>0){
            return "类型:"+productType.getTypeName()+"已经存在";
        }
        return null;
    }
}
