package com.bcxin.models.product.util;

import com.bcxin.common.mapper.JsonMapper;
import com.bcxin.models.product.dao.ProductTypeDao;
import com.bcxin.models.product.entity.ProductType;
import com.bcxin.spring.cache.CacheUtils;
import com.bcxin.spring.util.SpringContextHolder;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

/**
 *
 * 产品类型缓存类
 * Created by playboy on 2017/5/31.
 */
public class ProductTypeUtil {

    private static ProductTypeDao productTypeDao = SpringContextHolder.getBean(ProductTypeDao.class);

    public static final String CACHE_PRODUCT_TYPE_MAP = "ProductType";

    /**
     * 缓存数据 返回列表
     * @return
     */
    public static List<ProductType> getProductTypeList(String type){
        @SuppressWarnings("unchecked")
        Map<String, List<ProductType>> typeMap = (Map<String, List<ProductType>>) CacheUtils.get(CACHE_PRODUCT_TYPE_MAP);
        if (typeMap==null){
            typeMap = Maps.newHashMap();
            for (ProductType productType : productTypeDao.findList(new ProductType())){
                List<ProductType> productTypeList = typeMap.get(productType.getType());
                if (productTypeList != null){
                    productTypeList.add(productType);
                }else{
                    typeMap.put(productType.getType(), Lists.newArrayList(productType));
                }
            }
            CacheUtils.put(CACHE_PRODUCT_TYPE_MAP, typeMap);
        }
        List<ProductType> typeList = typeMap.get(type);
        if (typeList == null){
            typeList = Lists.newArrayList();
        }
        return typeList;
    }

    /**
     * 通过Id获取类型名称
     * @param value
     * @param type
     * @param defaultValue
     * @return
     */
    public static String getTypeName(String value, String type, String defaultValue){
        if (StringUtils.isNotBlank(type) && StringUtils.isNotBlank(value)){
            for (ProductType productType : getProductTypeList(type)){
                if (type.equals(productType.getType()) && value.equals(productType.getId())){
                    return productType.getTypeName();
                }
            }
        }
        return defaultValue;
    }

    /**
     * 返回字典列表（JSON）
     * @param type
     * @return
     */
    public static String getProductTypeListJson(String type){
        return JsonMapper.toJsonString(getProductTypeList(type));
    }

    /**
     * 保险类特殊 获取属性分类
     */
    public static String getProductInsTypeListJson(String type,Integer insType,String leve,String id){
        List<ProductType> list = Lists.newArrayList();
        if (StringUtils.isNotBlank(type) && null!=insType){
            for (ProductType productType : getProductTypeList(type)){
                if (type.equals(productType.getType()) && insType.equals(productType.getInsType()) && leve.equals(productType.getTypeLevel())){
                    if(id!=null){
                        if(productType.getParent()!=null && productType.getParent().getId().equals(StrUtil.toLong(id))) {
                            list.add(productType);
                        }
                    }else{
                        list.add(productType);
                    }
                }
            }
        }
        return JsonMapper.toJsonString(list);
    }

    /**
     * 获取产品分类
     */
    public static String getProductInsTypeListJson(String type,String leve,String id){
        List<ProductType> list = Lists.newArrayList();
        if (StringUtils.isNotBlank(type) ){
            for (ProductType productType : getProductTypeList(type)){
                if (type.equals(productType.getType()) && leve.equals(productType.getTypeLevel())){
                    if(id!=null){
                        if(productType.getParent()!=null && productType.getParent().getId().equals(StrUtil.toLong(id))) {
                            list.add(productType);
                        }
                    }else{
                        list.add(productType);
                    }
                }
            }
        }
        return JsonMapper.toJsonString(list);
    }


    /**
     * 清空缓存
     */
    public static void clear(){
        CacheUtils.remove(CACHE_PRODUCT_TYPE_MAP);
    }
}
