package com.bcxin.models.product.web;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.models.product.entity.ProductIns;
import com.bcxin.models.product.entity.ProductType;
import com.bcxin.models.product.service.ProductInsService;
import com.bcxin.models.product.util.ProductTypeUtil;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 保险产品
 * Created by playboy on 2017/6/1.
 */
@Controller
@RequestMapping("${systemPath}/productIns")
public class ProductInsController extends BaseController {
    @Autowired
    private ProductInsService productInsService;

    //list
    private static final String ADD = getViewPath("admin/product/product_ins_add");
    //view
    private static final String LIST = getViewPath("admin/product/product_ins_list");

    @RequiresPermissions("product:ins:list")
    @RequestMapping("")
    public String listView(Model model){
        return LIST;
    }


    @RequiresPermissions("product:ins:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(Model model,ProductIns productIns){
        Page<ProductIns> page =this.getPage();
        productInsService.findPage(page,productIns);
        return jsonPage(page);
    }

    @RequiresPermissions("product:ins:form")
    @RequestMapping("/form")
    public String form(Model model,@RequestParam(required=false)Long id){
        String flag = this.getParameter("modify");
        if(id!=null) {
            ProductIns productIns = productInsService.get(new ProductIns(id));
            List<ProductType> list = Lists.newArrayList();
            if(null!=productIns.getResponsibilityType()) {
                //查询所有父级
                ProductType.sortList(list, ProductTypeUtil.getProductTypeList(productIns.getProductType()), StrUtil.toLong(productIns.getResponsibilityType().getId()));
            }
            for(ProductType productType:list){
                model.addAttribute("responsibilityType"+productType.getTypeLevel(), productType.getId());
            }
            model.addAttribute("productIns", productIns);
        }
        model.addAttribute("modify",flag);
        return ADD;
    }

    @RequiresPermissions("product:ins:edit")
    @RequestMapping("/edit")
    @ResponseBody
    public AjaxResult edit(Model model, ProductIns productIns){
        String msg = beanValidator(productIns);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        msg = productInsService.checkData(productIns);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        if(null==productIns.getId()) {
            //默认为下架状态
            productIns.setStatus(0);
        }
        productInsService.save(productIns);
        return new AjaxResult(true,"保存成功",productIns.getId().toString());
    }

    @RequiresPermissions("product:ins:delete")
    @RequestMapping(value = "/delete/{id}",method = RequestMethod.DELETE)
    @ResponseBody
    public AjaxResult delete(Model model, @PathVariable Long id){
        productInsService.deleteById(id);
        return success(true);
    }

    /**
     * 产品上下架
     * @param model
     * @param id
     * @return
     */
    @RequiresPermissions("product:ins:upDown")
    @RequestMapping(value = "/upDown/{id}/{status}",method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult upDown(Model model, @PathVariable Long id,@PathVariable Integer status){
        productInsService.save(new ProductIns(id,status));
        return success(true,"操作成功");
    }
}
