package com.bcxin.models.product.web;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.core.dao.SysCompanyDao;
import com.bcxin.core.entity.SysCompany;
import com.bcxin.core.entity.SysUser;
import com.bcxin.core.service.SysCompanyService;
import com.bcxin.core.util.SysUserUtils;
import com.bcxin.models.agreement.dao.ProductRelationFeerateDao;
import com.bcxin.models.agreement.entity.Agreement;
import com.bcxin.models.agreement.entity.AgreementFee;
import com.bcxin.models.agreement.entity.ProductRelationFeerate;
import com.bcxin.models.agreement.service.AgreementFeeService;
import com.bcxin.models.agreement.service.AgreementService;
import com.bcxin.models.product.entity.ProductRelation;
import com.bcxin.models.product.service.ProductRelationService;
import com.bcxin.models.product.service.ProductService;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.util.toolbox.DateUtil;
import com.bcxin.util.toolbox.Str2Utils;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * 产品关联
 * Created by playboy on 2017/6/4.
 */
@Controller
@RequestMapping("${systemPath}/productRelation")
public class ProductRelationController extends BaseController {

    @Autowired
    ProductService productService;

    @Autowired
    ProductRelationService productRelationService;

    @Autowired
    SysCompanyService sysCompanyService;

    @Autowired
    AgreementFeeService agreementFeeService;

    @Autowired
    AgreementService agreementService;

    @Autowired
    ProductRelationFeerateDao productRelationFeerateDao;

    @Autowired
    SysCompanyDao sysCompanyDao;

    //选择产品页面
    private static final String SELECT_PRODUCT = getViewPath("admin/product/product_select");

    //已关联产品
    private static final String RELATION_PRODUCT = getViewPath("admin/product/product_relation");

    //已关联产品费率
    private static final String RELATION_PRODUCT_FEE = getViewPath("admin/product/product_relation_fee");

    //产品分享
    private static final String RELATION_PRODUCT_SHARE = getViewPath("admin/product/product_relation_share");

    /**
     * 下级分销售选择产品列表
     */
    private static final String PRODUCT_FEE_LIST_ONE = getViewPath("admin/product/product_change_list_one");

    /**
     * 下级分销商设置佣金
     */
    private static final String PRODUCT_FEE_DATA_ONE = getViewPath("admin/product/product_fee_data_one");

    /**
     * 分销商产品分配列表
     */
    private static final String PRODUCT_COMPANY_LIST_ONE = getViewPath("admin/product/product_company_list_one");

    /**
     * 查看已经分配的产品
     *
     * @param model
     * @return
     */
    @RequiresPermissions("product:relation:list")
    @RequestMapping(value = "/listView", method = RequestMethod.GET)
    public String listView(Model model) {
        String relationId = this.getParameter("relationId");
        model.addAttribute("relationId", relationId);
        model.addAttribute("type", this.getParameter("type"));
        model.addAttribute("leve", this.getParameter("leve"));
        return RELATION_PRODUCT;
    }

    /**
     * 查看已经分配的产品--分级费率
     *
     * @param model
     * @return
     */
    @RequiresPermissions("product:relation:list")
    @RequestMapping(value = "/listViewFee", method = RequestMethod.GET)
    public String listViewFee(Model model) {
        String productRelationId = this.getParameter("productRelationId");
        model.addAttribute("productRelationId", productRelationId);
        String relationId = this.getParameter("relationId");
        model.addAttribute("relationId", relationId);
        model.addAttribute("type", this.getParameter("type"));
        model.addAttribute("leve", this.getParameter("leve"));

        return RELATION_PRODUCT_FEE;
    }

    /**
     * 查看分享产品
     *
     * @param model
     * @return
     */
    @RequiresPermissions("product:relation:list:share")
    @RequestMapping(value = "/listViewShare", method = RequestMethod.GET)
    public String listViewShrea(Model model) {
        model.addAttribute("relationId", SysUserUtils.getUser().getSysCompany().getId());
        model.addAttribute("type", this.getParameter("type"));
        return RELATION_PRODUCT_SHARE;
    }

    /**
     * 查看已经分配的产品列表
     *
     * @param model
     * @return
     */
    @RequestMapping(value = "/listViewData")
    @ResponseBody
    @RequiresPermissions("product:relation:list")
    public String listViewData(Model model, String type, String relationId) {
        String likeValue = this.getParameter("likeValue");
        Map<String, Object> params = Maps.newHashMap();
        params.put("likeValue", likeValue);//模糊查询
        params.put("relationId", relationId);//关联ID
        params.put("type", type); //关联类型*/
        Page<Map<String, Object>> page = this.getPage();
        productService.getProductAssignedList(page, params);
        return jsonPage(page);
    }

    /**
     * 查看已经分配的产品列表--分级费率
     *
     * @param model
     * @return
     */
    @RequestMapping(value = "/listViewFeeData")
    @ResponseBody
    @RequiresPermissions("product:relation:list")
    public String listViewFeeData(Model model, String productRelationId, String relationId) {
        String likeValue = this.getParameter("likeValue");
        Map<String, Object> params = Maps.newHashMap();
        params.put("likeValue", likeValue);//模糊查询
        params.put("productRelationId", productRelationId);//关联ID
        params.put("relationId", relationId);//关联ID
        Page<Map<String, Object>> page = this.getPage();
        productService.getProductAssignedListFee(page, params);
        return jsonPage(page);
    }

    @RequiresPermissions("product:relation:list")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public String list(Model model, @RequestParam(required = true) String type, @RequestParam(required = true) String relationIds) {
        model.addAttribute("relationIds", relationIds);
        model.addAttribute("type", type);
        return SELECT_PRODUCT;
    }

    @RequestMapping(value = "/listData")
    @ResponseBody
    @RequiresPermissions("product:relation:list")
    public String listData(Model model, String relationIds, String type) {
        String likeValue = this.getParameter("likeValue");
        Map<String, Object> params = Maps.newHashMap();
        if (StrUtil.isNotBlank(relationIds)) {
            if (StrUtil.split(relationIds, ",").length > 0) {
                SysUser user = SysUserUtils.getUser();
                if (user.getSysCompany().getId().equals(StrUtil.toLong(StrUtil.split(relationIds, ",")[0]))) {
                    return null;
                }
                SysCompany sysCompany = sysCompanyService.get(StrUtil.toLong(StrUtil.split(relationIds, ",")[0]));
                if (null != sysCompany && sysCompany.getCompanyType().equals(SysCompany.COMPANY_TYPE_DISTRIBUTOR)
                        && null != sysCompany.getDistributionLevel() && sysCompany.getParent() != null && sysCompany.getParent().getId() != null) {
                    params.put("relationId", sysCompany.getParent().getId());
                }
            }
        }
        params.put("companyId", relationIds);
        params.put("type", type);
        params.put("likeValue", likeValue);
        params.put("status", "1");
        Page<Map<String, Object>> page = this.getPage();
        productService.getProductListPage(page, params);
        return jsonPage(page);
    }

    /**
     * 获取产品id和名称
     *
     * @param type
     * @return
     */
    @RequestMapping(value = "/getProductIdAndName", method = RequestMethod.GET)
    @ResponseBody
    public AjaxResult getProductIdAndName(String type, String companyId) {//List<Map<String,Object>>
        Map<String, Object> params = Maps.newHashMap();
        if (!companyId.equalsIgnoreCase("undefined")) {
            params.put("companyId", companyId);
            //        params.put("type",type);
        }
        //查询关联产品
        List<Map<String, Object>> retList = null;
        //如果是供应商，要改变换机构对应产品的取值方式
        if (SysCompany.COMPANY_TYPE_SUPPLIER_STRING.equalsIgnoreCase(type)) {
//                retList=productService.getProductIdAndNameBySupply(params);
            retList = sysCompanyService.getInsProductSupplyMap(params);
            retList.addAll(sysCompanyService.getInsProductMap(params));

        } else {
            retList = productService.getProductIdAndName(params);
        }
        return json(true, retList);
    }

//    /**
//     * 获取产品id和名称,区分供应商
//     * @param type
//     * @return
//     */
//    @RequestMapping(value = "/getProductIdAndName",method = RequestMethod.GET)
//    @ResponseBody
//    public AjaxResult  getProductIdAndNameByType(String type,String companyId){//List<Map<String,Object>>
//        Map<String,Object> params = Maps.newHashMap();
//        if(!companyId.equalsIgnoreCase("undefined"))
//            params.put("companyId",companyId);
//        params.put("type",type);
//        return json(true,productService.getProductIdAndName(params));
//    }

    /**
     * 添加关联
     *
     * @param model
     * @param productIds  必填
     * @param relationIds 必填
     * @param type        必填
     * @return
     */
    @RequiresPermissions("product:relation:edit")
    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult edit(Model model, @RequestParam(required = true) String productIds,
                           @RequestParam(required = true) String relationIds,
                           @RequestParam(required = true) Integer type) {
        productRelationService.checkSave(productIds, relationIds, type);
        return success(true, "操作成功");
    }


    /**
     * 删除关联
     */
    @RequiresPermissions("product:relation:delete")
    @RequestMapping(value = "/delete/{productId}/{relationId}", method = RequestMethod.DELETE)
    @ResponseBody
    @Transactional
    public AjaxResult delete(Model model, @PathVariable Long productId, @PathVariable Long relationId) {
        //根据机构查出协议
        Agreement agreement = agreementService.selectOne(new EntityWrapper<Agreement>().addFilter("company_id={0} and status=1", relationId));
        if (agreement != null) {
            agreementFeeService.delete(new EntityWrapper<AgreementFee>().addFilter("agreement_id={0} and product_id={1}", agreement.getId(), productId));
        }
        // 如果是分销商，查出本级和下级分销商列表，并从产品-机构关联中删除
        SysCompany sysCompany = sysCompanyDao.get(relationId);
        if (SysCompany.COMPANY_TYPE_DISTRIBUTOR.equals(sysCompany.getCompanyType())) {
            productRelationService.delete(new EntityWrapper<ProductRelation>().addFilter("product_id={0} and relation_id in (SELECT id FROM sys_company A  WHERE ( find_in_set( {1}, A.parent_ids ) OR A.id = {2} ) )", productId, relationId, relationId));
        } else {
            productRelationService.delete(new EntityWrapper<ProductRelation>().addFilter("product_id={0} and relation_id={1}", productId, relationId));
        }
        return success(true, "操作成功");
    }

    /**
     * 分配商品查询页面及资源方
     *
     * @return
     */
    @RequiresPermissions(value = {"product:list:productList:one"})
    @RequestMapping(value = "/list/companyToProductList")
    public String companyToProductList(Model model) {
        String companyId = this.getParameter("companyId");
        String level = this.getParameter("level");
        String parentId = this.getParameter("parentId");
        if (StrUtil.isNotBlank(companyId)) {
            List<Map<String, Object>> list = Lists.newArrayList();
            model.addAttribute("list", list);
            model.addAttribute("companyId", companyId);
            model.addAttribute("level", level);
            model.addAttribute("parentId", parentId);
        }
        return PRODUCT_COMPANY_LIST_ONE;
    }

    /**
     * 设置佣金 单个设置
     *
     * @param model
     * @return
     */
    @RequiresPermissions(value = {"product:list:productList:one"})
    @RequestMapping(value = "/list/openProductSetFeeOne")
    public String openProductSetFee(Model model) {
        String companyId = this.getParameter("companyId");
        String productIds = this.getParameter("productIds");
        if (StrUtil.isNotBlank(companyId)) {
            model.addAttribute("companyId", companyId);
            model.addAttribute("company", sysCompanyService.get(StrUtil.toLong(companyId)));
        }
        if (StrUtil.isNotBlank(productIds)) {
            List<Map<String, Object>> plst = Lists.newArrayList();
            model.addAttribute("plst", plst);
            model.addAttribute("productIds", productIds);
        }
        return PRODUCT_FEE_DATA_ONE;
    }

    /**
     * 打开选择产品表页面   单个设置
     *
     * @param model
     * @return
     */
    @RequiresPermissions(value = {"product:list:productList:one"})
    @RequestMapping(value = "/list/openProductList")
    public String openProductList(Model model) {
        String companyId = this.getParameter("companyId");
        String productIds = this.getParameter("productIds");
        if (StrUtil.isNotBlank(companyId)) {
            model.addAttribute("companyId", companyId);
        }
        if (StrUtil.isNotBlank(companyId)) {
            model.addAttribute("productIds", productIds);
        }
        //获取分销商等级,下级分销商
        String level = this.getParameter("level");
        String parentId = this.getParameter("parentId");
        List<Map<String, Object>> plst = Lists.newArrayList();
        if (SysCompany.DISTRIBUTIONLEVEL_TWO.equals(level) || SysCompany.DISTRIBUTIONLEVEL_THREE.equals(level)) {
            if (StrUtil.isNotBlank(parentId)) {
                plst = Lists.newArrayList();
            }
        } else {
            plst = Lists.newArrayList();
        }
        model.addAttribute("plst", plst);
        model.addAttribute("level", level);
        model.addAttribute("parentId", parentId);
        return PRODUCT_FEE_LIST_ONE;
    }

    /**
     * 添加关联
     *
     * @param model
     * @return
     */
    @RequiresPermissions("product:relation:saveFee")
    @RequestMapping(value = "/saveFee", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult saveFee(Model model, @RequestParam Long id, @RequestParam String fee, @RequestParam String productID) {
        //先获取自己的费率，然后获取比较设置的费率，不能大于自己的费率
        SysUser sysUser = SysUserUtils.getUser();
        ProductRelation pr = productRelationService.selectOne(new EntityWrapper().addFilter("relation_id={0} and product_id={1}", sysUser.getSysCompany().getId(), productID));
//        logger.info("获得的信息："+JSON.toJSONString(pr));
        if (pr == null || pr.getFee() == null || pr.getFee().compareTo(new BigDecimal("0")) == 0)
            return fail(false, "上级费率为空");
        if (pr.getFee().compareTo(new BigDecimal(fee)) == -1)
            return fail(false, "费率大于上级费率" + pr.getFee());//-1 小于   0 等于  1 大于

        ProductRelation productRelation = new ProductRelation();
        productRelation.setId(id);
        productRelation.setFee(Str2Utils.getBigDecimal(fee).setScale(2, BigDecimal.ROUND_HALF_UP));
        productRelation.setUpdateTIme(DateUtil.getTimestamp());
        productRelationService.updateById(productRelation);
        return success(true, "操作成功");
    }


    /**
     * 保存--分级费率
     *
     * @param model
     * @return
     */
    @RequiresPermissions("product:relation:saveFee")
    @RequestMapping(value = "/saveRangeFee", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult saveRangeFee(Model model, @RequestParam String fee, Long productRelationId, Long agreementFeeId, String taxScopeStart, String taxScopeEnd, String leve, String productId) {
        //  增加一个校验，当前设置的数组不能大于上级费率/固定金额
        //先获取自己的费率，然后获取比较设置的费率，不能大于自己所有上级的费率
        SysUser sysUser = SysUserUtils.getUser();

        //一级协议的费率
        AgreementFee agreementFee = agreementFeeService.selectOne(new EntityWrapper().addFilter("id={0}  ", agreementFeeId));
        if (new BigDecimal(fee).compareTo(agreementFee.getFeeRate()) > -1) {
            return fail(false, "不能大于一级分销商的费率");
        }
        //todo 数据库查询要优化下
        if (SysCompany.DISTRIBUTIONLEVEL_THREE.equalsIgnoreCase(leve)) {
//            ProductRelationFeerate feerateThree = new ProductRelationFeerate();
//            feerateThree.setProductRelationId(productRelationId);
//            feerateThree.setTaxScopeStart(new BigDecimal(taxScopeStart));
//            ProductRelationFeerate prfThree = productRelationFeerateDao.selectOne(feerateThree);
//            ProductRelation prThree = productRelationService.selectOne(new EntityWrapper().addFilter("id={0} ", prfThree.getProductRelationId()));
            ProductRelation pr = productRelationService.selectOne(new EntityWrapper().addFilter("relation_id={0} and product_id={1}", sysUser.getSysCompany().getId(), productId));
            ProductRelationFeerate feerate = new ProductRelationFeerate();
            feerate.setProductRelationId(pr.getId());
            feerate.setTaxScopeStart(new BigDecimal(taxScopeStart));
            feerate.setTaxScopeEnd(new BigDecimal(taxScopeEnd));
            ProductRelationFeerate prf = productRelationFeerateDao.selectOne(feerate);

            if (prf == null || prf.getFee() == null || prf.getFee().compareTo(new BigDecimal("0")) == 0)
                return fail(false, "上级费率为空");
            if (prf.getFee().compareTo(new BigDecimal(fee)) < 1)
                return fail(false, "费率大于或者等于上级费率" + prf.getFee());//-1 小于   0 等于  1 大于
        }

        //先删后加
        productRelationService.saveRangeFee(new BigDecimal(fee), productRelationId, agreementFeeId, new BigDecimal(taxScopeStart), new BigDecimal(taxScopeEnd));
        return success(true, "操作成功");
    }

}
