package com.bcxin.models.product.web;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.core.entity.SysCompany;
import com.bcxin.core.service.SysCompanyService;
import com.bcxin.models.product.entity.ProductRisk;
import com.bcxin.models.product.entity.ProductType;
import com.bcxin.models.product.service.ProductRelationService;
import com.bcxin.models.product.service.ProductRiskService;
import com.bcxin.models.product.util.ProductTypeUtil;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 风评产品
 * Created by playboy on 2017/6/1.
 */
@Controller
@RequestMapping("${systemPath}/productRisk")
public class ProductRiskController extends BaseController {
    @Autowired
    private ProductRiskService productRiskService;

    @Autowired
    private ProductRelationService productRelationService;

    @Autowired
    private SysCompanyService sysCompanyService;

    //list
    private static final String ADD = getViewPath("admin/product/product_risk_add");
    //view
    private static final String LIST = getViewPath("admin/product/product_risk_list");

    @RequiresPermissions("product:risk:list")
    @RequestMapping("")
    public String listView(Model model){
        return LIST;
    }


    @RequiresPermissions("product:risk:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(Model model,ProductRisk productRisk){
        Page<ProductRisk> page =this.getPage();
        productRiskService.findPage(page,productRisk);
        return jsonPage(page);
    }

    @RequiresPermissions("product:risk:form")
    @RequestMapping("/form")
    public String form(Model model,@RequestParam(required=false)Long id){
        String flag = this.getParameter("modify");
        if(id!=null) {
            ProductRisk productRisk = productRiskService.get(new ProductRisk(id));
            model.addAttribute("productRisk", productRisk);
            List<ProductType> list = Lists.newArrayList();
            if(null!=productRisk.getResponsibilityType()) {
                //查询所有父级
                ProductType.sortList(list, ProductTypeUtil.getProductTypeList(productRisk.getProductType()), StrUtil.toLong(productRisk.getResponsibilityType().getId()));
            }
            for(ProductType productType:list){
                model.addAttribute("responsibilityType"+productType.getTypeLevel(), productType.getId());
            }
            Map<String,Object> params = Maps.newHashMap();
            params.put("productId",id);
            params.put("type",4);//设置供应商
            List<Long> cList= productRelationService.getCompanyListId(params);
            SysCompany company = new SysCompany();
            company.setCompanyType(1);
            company.setEnterpriseType("7");
            List<SysCompany> companyList= sysCompanyService.getCompanyList(company);
            model.addAttribute("cList",cList);
            model.addAttribute("companyList",companyList);
        }
        model.addAttribute("modify",flag);
        return ADD;
    }

    @RequiresPermissions("product:risk:edit")
    @RequestMapping("/edit")
    @ResponseBody
    public AjaxResult edit(Model model, ProductRisk productRisk){
        String relationIds[] =getRequest().getParameterValues("supplier_ids");
        String msg = beanValidator(productRisk);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        msg = productRiskService.checkData(productRisk);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        if(!StrUtil.cheakIsRepeat(relationIds)){
            return success(false,"供应商不能重复！");
        }
        if(null==productRisk.getId()) {
            //默认为下架状态
            productRisk.setStatus(0);
        }
        productRiskService.save(productRisk);
        if(relationIds.length>0){
            for(String r:relationIds){
                if(StrUtil.isNotEmpty(r))
                productRelationService.checkSave(productRisk.getId()+"",r,4);//保存供应商与产品的关系
            }
        }
        return success(true,"操作成功");
    }

    @RequiresPermissions("product:risk:delete")
    @RequestMapping(value = "/delete/{id}",method = RequestMethod.DELETE)
    @ResponseBody
    public AjaxResult delete(Model model, @PathVariable Long id){
        productRiskService.deleteById(id);
        return success(true);
    }

    /**
     * 产品上下架
     * @param model
     * @param id
     * @return
     */
    @RequiresPermissions("product:risk:upDown")
    @RequestMapping(value = "/upDown/{id}/{status}",method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult upDown(Model model, @PathVariable Long id,@PathVariable Integer status){
        productRiskService.save(new ProductRisk(id,status));
        return success(true,"操作成功");
    }
}
