package com.bcxin.models.product.web;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.models.product.entity.ProductTrain;
import com.bcxin.models.product.entity.ProductType;
import com.bcxin.models.product.service.ProductTrainService;
import com.bcxin.models.product.util.ProductTypeUtil;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.util.toolbox.StrUtil;
import com.google.common.collect.Lists;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 培训产品
 * Created by playboy on 2017/6/1.
 */
@Controller
@RequestMapping("${systemPath}/productTrain")
public class ProductTrainController extends BaseController {
    @Autowired
    private ProductTrainService productTrainService;

    //list
    private static final String ADD = getViewPath("admin/product/product_train_add");
    //view
    private static final String LIST = getViewPath("admin/product/product_train_list");

    @RequiresPermissions("product:train:list")
    @RequestMapping("")
    public String listView(Model model){
        return LIST;
    }


    @RequiresPermissions("product:train:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(Model model,ProductTrain productTrain){
        Page<ProductTrain> page =this.getPage();
        productTrainService.findPage(page,productTrain);
        return jsonPage(page);
    }

    @RequiresPermissions("product:train:form")
    @RequestMapping("/form")
    public String form(Model model,@RequestParam(required=false)Long id){
        String flag = this.getParameter("modify");
        if(id!=null) {
            ProductTrain productTrain = productTrainService.get(new ProductTrain(id));
            model.addAttribute("productTrain", productTrain);
            List<ProductType> list = Lists.newArrayList();
            if(null!=productTrain.getResponsibilityType()) {
                //查询所有父级
                ProductType.sortList(list, ProductTypeUtil.getProductTypeList(productTrain.getProductType()), StrUtil.toLong(productTrain.getResponsibilityType().getId()));
            }
            for(ProductType productType:list){
                model.addAttribute("responsibilityType"+productType.getTypeLevel(), productType.getId());
            }
        }
        model.addAttribute("modify",flag);
        return ADD;
    }

    @RequiresPermissions("product:train:edit")
    @RequestMapping("/edit")
    @ResponseBody
    public AjaxResult edit(Model model, ProductTrain productTrain){
        String msg = beanValidator(productTrain);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        msg = productTrainService.checkData(productTrain);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        if(null==productTrain.getId()) {
            //默认为下架状态
            productTrain.setStatus(0);
        }
        productTrainService.save(productTrain);
        return success(true,"操作成功");
    }

    @RequiresPermissions("product:train:delete")
    @RequestMapping(value = "/delete/{id}",method = RequestMethod.DELETE)
    @ResponseBody
    public AjaxResult delete(Model model, @PathVariable Long id){
        productTrainService.deleteById(id);
        return success(true);
    }
    /**
     * 产品上下架
     * @param model
     * @param id
     * @return
     */
    @RequiresPermissions("product:train:upDown")
    @RequestMapping(value = "/upDown/{id}/{status}",method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult upDown(Model model, @PathVariable Long id,@PathVariable Integer status){
        productTrainService.save(new ProductTrain(id,status));
        return success(true,"操作成功");
    }
}
