package com.bcxin.models.product.web;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.models.product.entity.ProductIns;
import com.bcxin.models.product.entity.ProductRisk;
import com.bcxin.models.product.entity.ProductTrain;
import com.bcxin.models.product.entity.ProductType;
import com.bcxin.models.product.service.ProductInsService;
import com.bcxin.models.product.service.ProductRiskService;
import com.bcxin.models.product.service.ProductTrainService;
import com.bcxin.models.product.service.ProductTypeService;
import com.bcxin.models.product.util.ProductTypeUtil;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import com.bcxin.util.toolbox.StrUtil;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *
 *
 * Created by playboy on 2017/5/27.
 */
@Controller
@RequestMapping("${systemPath}/productType")
public class ProductTypeController extends BaseController {

    @Autowired
    private ProductTypeService productTypeService;

    @Autowired
    private ProductInsService productInsService;

    @Autowired
    private ProductRiskService productRiskService;

    @Autowired
    private ProductTrainService productTrainService;

    //list
    private static final String IP_ADD = getViewPath("admin/product/product_type_add");
    //view
    private static final String IP_LIST = getViewPath("admin/product/product_type_list");

    @RequiresPermissions("product:type:list")
    @RequestMapping("")
    public String listView(Model model){
        return IP_LIST;
    }


    @RequiresPermissions("product:type:list")
    @RequestMapping("/list")
    @ResponseBody
    public String listData(Model model,ProductType productType){
        Page<ProductType> page =this.getPage();
        productTypeService.findPage(page,productType);
        return jsonPage(page);
    }

    /**
     * 获取产品类型json  无限级联动
     *  instType 分类属性
     *  type   分类类型
     * @return
     */
    @RequestMapping("/getProductTypeJson")
    @ResponseBody
    public String getProductTypeJson(Integer insType,String type,String leve,String id){
        if(null==insType){
            return ProductTypeUtil.getProductInsTypeListJson(type,leve,id);
        }else {
            return ProductTypeUtil.getProductInsTypeListJson(type,insType,leve,id);
        }
    }

    @RequiresPermissions("product:type:form")
    @RequestMapping("/form")
    public String form(Model model,@RequestParam(required=false)Long id){
        String flag = this.getParameter("modify");
        if(id!=null) {
            ProductType productType = productTypeService.get(new ProductType(id));
            if("next".equals(flag)){
                ProductType parent = new ProductType();
                parent.setId(productType.getId());
                parent.setTypeName(productType.getTypeName());
                productType.setId(null);
                productType.setTypeName(null);
                productType.setParent(parent);
            }
            model.addAttribute("productType", productType);
        }
        model.addAttribute("modify",flag);
        return IP_ADD;
    }

    @RequiresPermissions("product:type:edit")
    @RequestMapping("/edit")
    @ResponseBody
    public AjaxResult edit(Model model, ProductType productType){
        String msg = beanValidator(productType);
        if(StrUtil.isNotBlank(msg)){
           return success(false,msg);
        }
        msg = productTypeService.checkData(productType);
        if(StrUtil.isNotBlank(msg)){
            return success(false,msg);
        }
        ProductTypeUtil.clear();
        productTypeService.save(productType);
        return success(true,"操作成功");
    }

    @RequiresPermissions("product:type:delete")
    @RequestMapping(value = "/delete/{id}",method = RequestMethod.DELETE)
    @ResponseBody
    public AjaxResult delete(Model model, @PathVariable Long id){
        //检查三个产品表是否存在关联，如果关联，那么就删除
        if(productInsService.selectOne(new EntityWrapper<ProductIns>().addFilter("responsibility_type = {0}",id))!=null)
            return fail(false,"该产品分类还在使用，不可删除");
        if(productRiskService.selectOne(new EntityWrapper<ProductRisk>().addFilter("responsibility_type = {0}",id))!=null)
            return fail(false,"该产品分类还在使用，不可删除");
        if(productTrainService.selectOne(new EntityWrapper<ProductTrain>().addFilter("responsibility_type = {0}",id))!=null)
            return fail(false,"该产品分类还在使用，不可删除");

        //如果有分类parent_id等于该分类的id，那么就表示该分类有子类，不可以删除
        if(productTypeService.selectOne(new EntityWrapper<ProductType>().addFilter(" parent_id = {0}",id))!=null)
            return fail(false,"该分类还有子分类，不可删除");
        ProductType pt = new ProductType();
        pt.setId(id);
        pt.setStatus(2);
        productTypeService.updateById(pt);
        ProductTypeUtil.clear();
        return success(true);
    }

    /**
     * 获取父级可选类型
     * @return
     */
    @RequestMapping(value = "/getProductTypeByLevel",method = RequestMethod.POST)
    @ResponseBody
    public List<ProductType> getProductTypeByLevel(@RequestParam(required = true) String type,@RequestParam(required = true)String leve){
        List<ProductType> list = productTypeService.selectList(new EntityWrapper<ProductType>().addFilter("type = {0} and type_level = {1}",type,leve));
        return list;
    }
}
