package com.bcxin.models.statements.controller;

import com.bcxin.common.dto.AjaxResult;
import com.bcxin.core.base.web.BaseController;
import com.bcxin.core.entity.SysUser;
import com.bcxin.core.util.SysUserUtils;
import com.bcxin.models.statements.controller.valid.StatementsCalcBatchGroup;
import com.bcxin.models.statements.controller.valid.StatementsCalcGroup;
import com.bcxin.models.statements.dto.StatementsDto;
import com.bcxin.models.statements.service.StatementsService;
import com.bcxin.mybatisplus.plugins.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;


/**
 * 结算单
 *
 * @author wangjianjun
 * @date 2020/3/4
 */
@Controller
@RequestMapping("${systemPath}/statementsForm")
public class StatementsController extends BaseController {

    @Autowired
    private StatementsService statementsService;

    //新建结算单页面
    private static final String ADD = getViewPath("admin/statements/statements_form_add");
    //批量新建结算单页面
    private static final String ADD_BATCH = getViewPath("admin/statements/statements_form_add_batch");
    //结算单列表页
    private static final String LIST = getViewPath("admin/statements/statements_form_list");
    //账单明细页面
    private static final String SUB_LIST = getViewPath("admin/bill/bill_form_list");


    /**
     * 结算单列表页
     *
     * @param model
     * @return
     */
//    @RequiresPermissions("statements:form:list")
    @RequestMapping("")
    public String listView(Model model) {
        return LIST;
    }
    //    @RequiresPermissions("statements:form:list")

    /**
     * 账单明细页面
     *
     * @param model
     * @return
     */
    @RequestMapping(value = "/subListView", method = RequestMethod.GET)
    public String subListView(Model model) {
        String settleId = this.getParameter("settleId");
        model.addAttribute("settleId", settleId);
        return SUB_LIST;
    }

    /**
     * 新建结算单页面
     *
     * @param model
     * @return
     */
    @RequestMapping("/add")
    public String add(Model model) {
        return ADD;
    }

    /**
     * 批量新建结算单页面
     *
     * @param model
     * @return
     */
    @RequestMapping("/addBatch")
    public String addBatch(Model model) {
        return ADD_BATCH;
    }

//以上为页面

    /**
     * 结算单列表
     *
     * @author wangjianjun
     * @date 2020/3/4
     */
    @RequestMapping("/list")
    @ResponseBody
    public String listData(StatementsDto dto) {
        Page page = this.getPage();
        statementsService.findPage(page, dto);
        return jsonPage(page);
    }

    /**
     * 删除结算单
     *
     * @param id 结算单id
     * @return
     */
    @RequestMapping(value = "/delete/{id}", method = RequestMethod.DELETE)
    @ResponseBody
    public AjaxResult deleteStatements(@PathVariable("id") Long id) {
        try {
            statementsService.deleteStatements(id);
            return success(true, "删除成功");
        } catch (Exception e) {
            logger.error("删除结算单出错", e);
            return fail(false, "删除失败! "+e.getMessage());

        }
    }

    /**
     * 变更结算单状态
     *
     * @param id     结算单ID
     * @param status 结算单状态，1是已结算，0是未结算，默认0
     * @return
     */
    @RequestMapping(value = "/modifyStatus/{id}/{status}", method = RequestMethod.PUT)
    @ResponseBody
    public AjaxResult modifyStatus(@PathVariable("id") Long id, @PathVariable("status") String status) {
        //已结算变成未结算，未结算变成已结算
        try {
            statementsService.modifyStatus(id, status);
            return success(true, "修改成功");
        } catch (Exception e) {
            logger.error("删除结算单出错", e);
            return fail(false, "修改失败");
        }
    }

    /**
     * 计算结算单
     *
     * @return
     */
    @RequestMapping("/calc")
    @ResponseBody
    public AjaxResult calc(@RequestBody @Validated(value = {StatementsCalcGroup.class}) StatementsDto dto, BindingResult bindingResult) {
        /**校验开始**/
        //协议类型、公司ID，时间段，产品类型四个值必须有
        if (bindingResult.hasErrors()) {
            String errorInfo = "";
            List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
            for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";
            }
            throw new RuntimeException(errorInfo);//返回校验错误
        }
        /**校验结束**/

        dto.setStartDate(dto.getStartDate() + "-01");//当月第一天
        dto.setEndDate(dto.getEndDate() + "-01");//sql里取当月最后一天
        List<StatementsDto> statementsDtos = statementsService.calc(dto);
        StatementsDto statementsDto = statementsDtos.get(0);
        statementsDto.setTotalCost(statementsDto.getTotalFixedCost().add(statementsDto.getTotalVariableCost()));
        return json(true, statementsDto);
    }

    /**
     * 批量计算结算单
     *
     * @return
     */
    @RequestMapping("/calcBatch")
    @ResponseBody
    public AjaxResult calcBatch(@RequestBody @Validated(value = {StatementsCalcBatchGroup.class}) StatementsDto dto, BindingResult bindingResult) {
        /**校验开始**/
        if (bindingResult.hasErrors()) {
            String errorInfo = "";
            List<FieldError> errors = bindingResult.getFieldErrors();//获取字段参数不合法的错误集合
            for (FieldError error : errors) {
//                    errorInfo = errorInfo + "[" + error.getField() + " " + error.getDefaultMessage() + "]";//注释掉，字段不能暴露
                errorInfo = errorInfo + "[" + error.getDefaultMessage() + "]";
            }
            throw new RuntimeException(errorInfo);//返回校验错误
        }
        /**校验结束**/

        dto.setStartDate(dto.getStartDate() + "-01");//当月第一天
        dto.setEndDate(dto.getEndDate() + "-01");//sql里取当月最后一天
//        dto.setIsBatch("1");//开启批量

        List<StatementsDto> statementsDtos = statementsService.calc(dto);
        StatementsDto statementsDto = statementsDtos.get(0);
        statementsDto.setTotalCost(statementsDto.getTotalFixedCost().add(statementsDto.getTotalVariableCost()));
        return json(true, statementsDto);
    }

    /**
     * 生成结算单
     *
     * @return
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult save(@RequestBody StatementsDto dto) {
        try {
            SysUser user = SysUserUtils.getUser();
            dto.setCreateBy(user.getId());//创建人
            dto.setFeeCycle(dto.getStartDate() + "~" + dto.getEndDate());
            statementsService.statementsSave(dto);
            return success(true, "生成结算单成功");
        } catch (Exception e) {
            logger.error("生成结算单失败", e);
            return fail(false, "生成结算单失败:" + e.getMessage());
        }
    }

    /**
     * 生成结算单-批量--整个结算过程当成一个事务进行处理。暂时这样。
     *
     * @return
     */
    @RequestMapping(value = "/saveBatch", method = RequestMethod.POST)
    @ResponseBody
    public AjaxResult saveBatch(@RequestBody StatementsDto dto) {
        try {
            //获得用户
            SysUser user = SysUserUtils.getUser();
            String startDateTemp = dto.getStartDate();
            String endDateTemp = dto.getEndDate();
            dto.setIsBatch("1");//开启批量
            dto.setStartDate(dto.getStartDate() + "-01");//当月第一天
            dto.setEndDate(dto.getEndDate() + "-01");//sql里取当月最后一天
            List<StatementsDto> statementsDtos = statementsService.calc(dto);
            statementsDtos = statementsDtos.stream().filter(s -> {
                s.setTotalCost(s.getTotalFixedCost().add(s.getTotalVariableCost()));//成本总额=固定成本+变动成本
                s.setCreateBy(user.getId());//创建人
                s.setStartDate(startDateTemp);
                s.setEndDate(endDateTemp);
                return true;
            }).collect(Collectors.toList());
            statementsService.statementsSaveBatch(statementsDtos);
            return success(true, "批量生成结算单成功");
        } catch (Exception e) {
            logger.error("批量生成结算单失败", e);
            return fail(false, "批量生成结算单失败!  " + e.getMessage());
        }
    }

}
