package com.bcxin.models.statements.service.impl;


import com.bcxin.core.base.service.impl.BaseServiceImpl;
import com.bcxin.core.entity.SysUser;
import com.bcxin.core.util.SysUserUtils;
import com.bcxin.models.bill.dao.NewBillDao;
import com.bcxin.models.bill.dto.BillDto;
import com.bcxin.models.statements.dao.StatementsDao;
import com.bcxin.models.statements.dto.StatementsDto;
import com.bcxin.models.statements.entity.Statements;
import com.bcxin.models.statements.service.StatementsService;
import com.bcxin.mybatisplus.mapper.EntityWrapper;
import com.bcxin.mybatisplus.plugins.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * 订单管理
 * Created by playboy on 2017/6/7.
 */
@Service
public class StatementsServiceImpl extends BaseServiceImpl<StatementsDao, Statements> implements StatementsService {
    @Autowired
    private StatementsDao statementsDao;
    @Autowired
    private NewBillDao newBillDao;

    @Override
    public Page findPage(Page page, StatementsDto dto) {
//        this.dataScopeFilter(orderForm,"b.supplier_id");
//        StatementsDto dto= StatementsDto();
//        PageHelper.startPage(0,10);
        page.setRecords(statementsDao.page(page,dto));
        return page;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteStatements(Long id) throws Exception {
        //  只能未结算才能删除
        Statements statements = new Statements();
        statements.setId(id);
        Integer count = statementsDao.selectCount(new EntityWrapper<Statements>().eq("id", id));
        if (count == null || count < 1) {
            throw new RuntimeException("只有未结算才能删除");
        }
        newBillDao.cleanSettleId(id);//清除结算单编号
        //删除结算单
        statementsDao.delete(new EntityWrapper<Statements>().addFilter("id={0} and status=0", id));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void modifyStatus(Long id, String status) {
        Statements statements = new Statements();
        //状态反转
        if ("0".equalsIgnoreCase(status)) {
            status = "1";
            statements.setSettleTime(new Date());
            SysUser user = SysUserUtils.getUser();
            statements.setUpdateBy(user.getId());//结算人
        } else {
            status = "0";
            statements.setSettleTime(null);
            statements.setUpdateBy(null);//清空结算人
        }
        Map param = new HashMap();
        param.put("settleId", id);
        param.put("verification", status);
        newBillDao.modifyVerification(param);
        //修改结算表
        statements.setId(id);
        statements.setStatus(status);
        statementsDao.updateByPrimaryKeySelectiveModify(statements);
    }

    @Override
    public List<StatementsDto> calc(StatementsDto map) {
        return statementsDao.calc(map);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void statementsSave(StatementsDto dto) throws Exception {
        dto.setStartDate(dto.getStartDate() + "-01");//当月第一天
        dto.setEndDate(dto.getEndDate() + "-01");//sql里取当月最后一天
        //结算单表 直接插入
//        Long id = IdWorker.getId();
//        dto.setId(id);
        statementsDao.insertSelective(dto);
        //账单表 先捞出，然后塞入结算单号
        //   如果捞到有结算单号的，抛出异常
        List<BillDto> billDtos = statementsDao.getBillList(dto);
        for (BillDto billDto : billDtos) {
            if (billDto.getSettleId() != null) {
                throw new Exception("该周期已经存在结算单，不可重复结算");
            }
        }
        List<Long> idList = billDtos.stream().map(s -> s.getId()).collect(Collectors.toList());
        newBillDao.updateSettleIdBatch(idList, dto.getId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void statementsSaveBatch(List<StatementsDto> statementsDtos) throws Exception {
        for (StatementsDto dto : statementsDtos) {
            dto.setStartDate(dto.getStartDate() + "-01");//当月第一天
            dto.setEndDate(dto.getEndDate() + "-01");//sql里取当月最后一天
            //结算单表 直接插入
//            Long id = IdWorker.getId();
//            dto.setId(id);
            statementsDao.insertSelective(dto);
            //账单表 先捞出，然后塞入结算单号
            //   如果捞到有结算单号的，抛出异常
            List<BillDto> billDtos = statementsDao.getBillList(dto);
            for (BillDto billDto : billDtos) {
                if (billDto.getSettleId() != null) {
                    throw new Exception("该周期已经存在结算单，不可重复结算");
                }
            }
            List<Long> idList = billDtos.stream().map(s -> s.getId()).collect(Collectors.toList());
            newBillDao.updateSettleIdBatch(idList, dto.getId());
        }
    }
}
